@@ -169,11 +169,29 @@ less(Type t, Ftor f) {
   return less_t<Type, Ftor>(t, f);
 }
 
+template <typename FtorA, typename FtorB>
+struct less2_t : public std::binary_function<typename FtorA::argument_type, typename FtorB::argument_type, bool> {
+  less2_t(FtorA f_a, FtorB f_b) : m_f_a(f_a), m_f_b(f_b) {}
+
+  bool operator () (typename FtorA::argument_type a, typename FtorB::argument_type b) {
+    return m_f_a(a) < m_f_b(b);
+  }
+
+  FtorA m_f_a;
+  FtorB m_f_b;
+};
+
+template <typename FtorA, typename FtorB>
+inline less2_t<FtorA, FtorB>
+less2(FtorA f_a, FtorB f_b) {
+  return less2_t<FtorA,FtorB>(f_a,f_b);
+}
+
 template <typename Type, typename Ftor>
-struct _greater {
+struct greater_t : public std::unary_function<typename Ftor::argument_type, bool> {
   typedef bool result_type;
 
-  _greater(Type t, Ftor f) : m_t(t), m_f(f) {}
+  greater_t(Type t, Ftor f) : m_t(t), m_f(f) {}
 
   template <typename Arg>
   bool operator () (Arg& a) {
@@ -185,9 +203,9 @@ struct _greater {
 };
 
 template <typename Type, typename Ftor>
-inline _greater<Type, Ftor>
+inline greater_t<Type, Ftor>
 greater(Type t, Ftor f) {
-  return _greater<Type, Ftor>(t, f);
+  return greater_t<Type, Ftor>(t, f);
 }
 
 template <typename Type, typename Ftor>
@@ -260,12 +278,12 @@ on(Src s, Dest d) {
 
 // Creates a functor for accessing a member.
 template <typename Class, typename Member>
-struct mem_ptr_ref_t : public std::unary_function<Class&, Member&> {
+struct mem_ptr_ref_t : public std::unary_function<const Class&, const Member&> {
   mem_ptr_ref_t(Member Class::*m) : m_member(m) {}
 
-  Member& operator () (Class& c) {
-    return c.*m_member;
-  }
+//   Member& operator () (Class& c) {
+//     return c.*m_member;
+//   }
 
   const Member& operator () (const Class& c) {
     return c.*m_member;
@@ -442,13 +460,17 @@ template <typename Object, typename Ret, typename Arg1, typename Arg2>
 class mem_fun2 : public std::binary_function<Arg1, Arg2, Ret> {
 public:
   typedef Ret (Object::*Function)(Arg1, Arg2);
+  typedef Object object_type;
 
   mem_fun2() : m_object(NULL) {}
   mem_fun2(Object* o, Function f) : m_object(o), m_function(f) {}
 
-  bool is_valid() const { return m_object; }
+  bool         is_valid() const { return m_object; }
 
-  Ret operator () (Arg1 a1, Arg2 a2) { return (m_object->*m_function)(a1, a2); }
+  object_type*       object() { return m_object; }
+  const object_type* object() const { return m_object; }
+
+  Ret          operator () (Arg1 a1, Arg2 a2) { return (m_object->*m_function)(a1, a2); }
   
 private:
   Object* m_object;
