@@ -39,7 +39,9 @@
 #include <functional>
 
 #include "torrent/exceptions.h"
+
 #include "chunk.h"
+#include "chunk_iterator.h"
 
 namespace torrent {
 
@@ -53,8 +55,25 @@ Chunk::clear() {
   std::for_each(begin(), end(), std::mem_fun_ref(&ChunkPart::clear));
 
   m_size = 0;
-  m_prot = 0;
-  Base::clear();
+  m_prot = ~0;
+  base_type::clear();
+}
+
+// Each add calls vector's reserve adding 1. This should keep
+// the size of the vector at exactly what we need. Though it
+// will require a few more cycles, it won't matter as we only
+// rarely have more than 1 or 2 nodes.
+//
+// If the user knows how many chunk parts he is going to add, then he
+// may call reserve prior to this.
+void
+Chunk::push_back(value_type::mapped_type mapped, const MemoryChunk& c) {
+  m_prot &= c.get_prot();
+
+  reserve(size() + 1);
+  base_type::insert(end(), ChunkPart(mapped, c, m_size));
+
+  m_size += c.size();
 }
 
 Chunk::iterator
@@ -73,7 +92,7 @@ Chunk::at_position(uint32_t pos) {
   return itr;
 }
 
-Chunk::MemoryArea
+Chunk::data_type
 Chunk::at_memory(uint32_t offset, iterator part) {
   if (part == end())
     throw internal_error("Chunk::at_memory(...) reached end.");
@@ -86,24 +105,7 @@ Chunk::at_memory(uint32_t offset, iterator part) {
 
   offset -= part->position();
 
-  return MemoryArea(part->chunk().begin() + offset, part->size() - offset);
-}
-
-// Each add calls vector's reserve adding 1. This should keep
-// the size of the vector at exactly what we need. Though it
-// will require a few more cycles, it won't matter as we only
-// rarely have more than 1 or 2 nodes.
-void
-Chunk::push_back(const MemoryChunk& c) {
-  if (empty())
-    m_prot = c.get_prot();
-  else
-    m_prot &= c.get_prot();
-
-  //Base::reserve(Base::size() + 1);
-  Base::insert(end(), ChunkPart(c, m_size));
-
-  m_size += c.size();
+  return data_type(part->chunk().begin() + offset, part->size() - offset);
 }
 
 uint32_t
@@ -136,4 +138,78 @@ Chunk::sync(int flags) {
   return success;
 }
 
+// Consider using uint32_t returning first mismatch or length if
+// matching.
+bool
+Chunk::to_buffer(void* buffer, uint32_t position, uint32_t length) {
+  if (position + length > m_size)
+    throw internal_error("Chunk::to_buffer(...) position + length > m_size.");
+
+  if (length == 0)
+    return true;
+
+  Chunk::data_type data;
+  ChunkIterator itr(this, position, position + length);
+
+  do {
+    data = itr.data();
+    std::memcpy(buffer, data.first, data.second);
+
+    buffer = static_cast<char*>(buffer) + data.second;
+
+  } while (itr.used(data.second));
+  
+  return true;
+}
+
+// Consider using uint32_t returning first mismatch or length if
+// matching.
+bool
+Chunk::from_buffer(const void* buffer, uint32_t position, uint32_t length) {
+  if (position + length > m_size)
+    throw internal_error("Chunk::from_buffer(...) position + length > m_size.");
+
+  if (length == 0)
+    return true;
+
+  Chunk::data_type data;
+  ChunkIterator itr(this, position, position + length);
+
+  do {
+    data = itr.data();
+    std::memcpy(data.first, buffer, data.second);
+
+    buffer = static_cast<const char*>(buffer) + data.second;
+
+  } while (itr.used(data.second));
+  
+  return true;
+}
+
+// Consider using uint32_t returning first mismatch or length if
+// matching.
+bool
+Chunk::compare_buffer(const void* buffer, uint32_t position, uint32_t length) {
+  if (position + length > m_size)
+    throw internal_error("Chunk::compare_buffer(...) position + length > m_size.");
+
+  if (length == 0)
+    return true;
+
+  Chunk::data_type data;
+  ChunkIterator itr(this, position, position + length);
+
+  do {
+    data = itr.data();
+
+    if (std::memcmp(data.first, buffer, data.second) != 0)
+      return false;
+
+    buffer = static_cast<const char*>(buffer) + data.second;
+
+  } while (itr.used(data.second));
+  
+  return true;
+}
+
 }
