@@ -44,26 +44,28 @@ namespace torrent {
 
 class Chunk : private std::vector<ChunkPart> {
 public:
-  typedef std::vector<ChunkPart>    Base;
-  typedef std::pair<void*,uint32_t> MemoryArea;
+  typedef std::vector<ChunkPart>    base_type;
+  typedef std::pair<void*,uint32_t> data_type;
 
-  using Base::value_type;
+  using base_type::value_type;
 
-  using Base::iterator;
-  using Base::reverse_iterator;
-  using Base::size;
-  using Base::empty;
+  using base_type::iterator;
+  using base_type::reverse_iterator;
+  using base_type::size;
+  using base_type::empty;
+  using base_type::reserve;
 
-  using Base::begin;
-  using Base::end;
-  using Base::rbegin;
-  using Base::rend;
+  using base_type::begin;
+  using base_type::end;
+  using base_type::rbegin;
+  using base_type::rend;
 
-  Chunk() : m_size(0), m_prot(0) {}
+  Chunk() : m_size(0), m_prot(~0) {}
   ~Chunk() { clear(); }
 
   bool                is_all_valid() const;
 
+  // All permissions are set for empty chunks.
   bool                is_readable() const             { return m_prot & MemoryChunk::prot_read; }
   bool                is_writable() const             { return m_prot & MemoryChunk::prot_write; }
   bool                has_permissions(int prot) const { return !(prot & ~m_prot); }
@@ -72,16 +74,22 @@ public:
 
   void                clear();
 
-  iterator            at_position(uint32_t pos);
-  MemoryArea          at_memory(uint32_t offset, iterator part);
+  void                push_back(value_type::mapped_type mapped, const MemoryChunk& c);
 
-  void                push_back(const MemoryChunk& c);
+  iterator            at_position(uint32_t pos);
+  iterator            at_position(uint32_t pos, iterator itr);
+
+  data_type           at_memory(uint32_t offset, iterator part);
 
   // Check how much of the chunk is incore from pos.
   uint32_t            incore_length(uint32_t pos);
 
   bool                sync(int flags);
 
+  bool                to_buffer(void* buffer, uint32_t position, uint32_t length);
+  bool                from_buffer(const void* buffer, uint32_t position, uint32_t length);
+  bool                compare_buffer(const void* buffer, uint32_t position, uint32_t length);
+
 private:
   Chunk(const Chunk&);
   void operator = (const Chunk&);
@@ -90,6 +98,14 @@ private:
   int                 m_prot;
 };
 
+inline Chunk::iterator
+Chunk::at_position(uint32_t pos, iterator itr) {
+  while (itr != end() && itr->position() + itr->size() <= pos)
+    itr++;
+
+  return itr;
+}
+
 }
 
 #endif
