@@ -61,11 +61,11 @@ AvailableList::pop_random() {
 }
 
 void
-AvailableList::push_back(const SocketAddress& sa) {
-  if (std::find(begin(), end(), sa) != end())
+AvailableList::push_back(const rak::socket_address* sa) {
+  if (std::find(begin(), end(), *sa) != end())
     return;
 
-  Base::push_back(sa);
+  Base::push_back(*sa);
 }
 
 void
@@ -87,7 +87,7 @@ AvailableList::insert(AddressList* l) {
 }
 
 void
-AvailableList::erase(const SocketAddress& sa) {
+AvailableList::erase(const rak::socket_address& sa) {
   iterator itr = std::find(begin(), end(), sa);
 
   if (itr != end())
