@@ -37,7 +37,7 @@
 #include "config.h"
 
 #include <algorithm>
-#include <cstdlib>
+#include <stdlib.h>
 #include <rak/functional.h>
 
 #include "protocol/peer_chunks.h"
@@ -54,7 +54,8 @@ ChunkSelector::initialize(Bitfield* bf, ChunkStatistics* cs) {
   m_position = invalid_chunk;
   m_statistics = cs;
 
-  m_bitfield.resize(bf->size_bits());
+  m_bitfield.set_size_bits(bf->size_bits());
+  m_bitfield.allocate();
   std::transform(bf->begin(), bf->end(), m_bitfield.begin(), rak::invert<Bitfield::value_type>());
   m_bitfield.update();
 
@@ -78,7 +79,7 @@ ChunkSelector::update_priorities() {
   m_sharedQueue.clear();
 
   if (m_position == invalid_chunk)
-    m_position = std::rand() % size();
+    m_position = random() % size();
 
   advance_position();
 }
@@ -203,8 +204,8 @@ ChunkSelector::received_have_chunk(PeerChunks* pc, uint32_t index) {
 }
 
 bool
-ChunkSelector::search_linear(const Bitfield* bf, rak::partial_queue* pq, PriorityRanges* ranges, uint32_t first, uint32_t last) {
-  PriorityRanges::iterator itr = ranges->find(first);
+ChunkSelector::search_linear(const Bitfield* bf, rak::partial_queue* pq, priority_ranges* ranges, uint32_t first, uint32_t last) {
+  priority_ranges::iterator itr = ranges->find(first);
 
   while (itr != ranges->end() && itr->first < last) {
 
