@@ -41,78 +41,63 @@
 #include <vector>
 #include <rak/functional.h>
 
+#include "torrent/transfer_list.h"
+
 namespace torrent {
 
 class BitField;
+class Block;
+class BlockList;
+class BlockTransfer;
 class Content;
 class DownloadMain;
-class DelegatorChunk;
-class DelegatorReservee;
-class DelegatorPiece;
 class Piece;
 class PeerChunks;
+class PeerInfo;
 class ChunkSelector;
 
 class Delegator {
 public:
-  typedef std::vector<DelegatorChunk*>                              Chunks;
   typedef rak::mem_fun1<ChunkSelector, void, uint32_t>              SlotChunkIndex;
   typedef rak::mem_fun2<ChunkSelector, uint32_t, PeerChunks*, bool> SlotChunkFind;
-  typedef rak::mem_fun1<DownloadMain, void, uint32_t>               SlotChunkDone;
   typedef rak::const_mem_fun1<Content, uint32_t, uint32_t>          SlotChunkSize;
 
   static const unsigned int block_size = 1 << 14;
 
   Delegator() : m_aggressive(false) { }
-  ~Delegator() { clear(); }
 
-  void               clear();
+  TransferList*       transfer_list()                     { return &m_transfers; }
+  const TransferList* transfer_list() const               { return &m_transfers; }
 
-  DelegatorReservee* delegate(PeerChunks* peerChunks, int affinity);
-
-  void               finished(DelegatorReservee& r);
-
-  void               done(unsigned int index);
-  void               redo(unsigned int index);
-
-  Chunks&            get_chunks()                         { return m_chunks; }
+  BlockTransfer*     delegate(PeerChunks* peerChunks, int affinity);
 
   bool               get_aggressive()                     { return m_aggressive; }
   void               set_aggressive(bool a)               { m_aggressive = a; }
 
-  void               slot_chunk_enable(SlotChunkIndex s)  { m_slotChunkEnable = s; }
-  void               slot_chunk_disable(SlotChunkIndex s) { m_slotChunkDisable = s; }
   void               slot_chunk_find(SlotChunkFind s)     { m_slotChunkFind = s; }
-  void               slot_chunk_done(SlotChunkDone s)     { m_slotChunkDone = s; }
   void               slot_chunk_size(SlotChunkSize s)     { m_slotChunkSize = s; }
 
   // Don't call this from the outside.
-  DelegatorPiece*    delegate_piece(DelegatorChunk* c);
-  DelegatorPiece*    delegate_aggressive(DelegatorChunk* c, uint16_t* overlapped);
+  Block*             delegate_piece(BlockList* c, const PeerInfo* peerInfo);
+  Block*             delegate_aggressive(BlockList* c, uint16_t* overlapped, const PeerInfo* peerInfo);
 
 private:
   // Start on a new chunk, returns .end() if none possible. bf is
   // remote peer's bitfield.
-  DelegatorPiece*    new_chunk(PeerChunks* pc, bool highPriority);
-  DelegatorPiece*    find_piece(const Piece& p);
+  Block*             new_chunk(PeerChunks* pc, bool highPriority);
 
-  DelegatorPiece*    delegate_seeder(PeerChunks* peerChunks);
+  Block*             delegate_seeder(PeerChunks* peerChunks);
 
-  bool               all_finished(int index);
+  TransferList       m_transfers;
 
   bool               m_aggressive;
 
-  Chunks             m_chunks;
-
   // Propably should add a m_slotChunkStart thing, which will take
   // care of enabling etc, and will be possible to listen to.
-  SlotChunkIndex     m_slotChunkEnable;
-  SlotChunkIndex     m_slotChunkDisable;
   SlotChunkFind      m_slotChunkFind;
-  SlotChunkDone      m_slotChunkDone;
   SlotChunkSize      m_slotChunkSize;
 };
 
-} // namespace torrent
+}
 
-#endif // LIBTORRENT_DELEGATOR_H
+#endif
