@@ -71,12 +71,13 @@ DownloadMain::DownloadMain() :
   m_connectionList = new ConnectionList(m_info);
   m_chokeManager = new ChokeManager(m_connectionList);
 
-  m_delegator.slot_chunk_enable(rak::make_mem_fun(m_chunkSelector, &ChunkSelector::using_index));
-  m_delegator.slot_chunk_disable(rak::make_mem_fun(m_chunkSelector, &ChunkSelector::not_using_index));
   m_delegator.slot_chunk_find(rak::make_mem_fun(m_chunkSelector, &ChunkSelector::find));
-  m_delegator.slot_chunk_done(rak::make_mem_fun(this, &DownloadMain::receive_chunk_done));
   m_delegator.slot_chunk_size(rak::make_mem_fun(&m_content, &Content::chunk_index_size));
 
+  m_delegator.transfer_list()->slot_canceled(std::bind1st(std::mem_fun(&ChunkSelector::not_using_index), m_chunkSelector));
+  m_delegator.transfer_list()->slot_queued(std::bind1st(std::mem_fun(&ChunkSelector::using_index), m_chunkSelector));
+  m_delegator.transfer_list()->slot_completed(std::bind1st(std::mem_fun(&DownloadMain::receive_chunk_done), this));
+
   m_taskTrackerRequest.set_slot(rak::mem_fn(this, &DownloadMain::receive_tracker_request));
 
   m_chunkList->slot_create_chunk(rak::make_mem_fun(&m_content, &Content::create_chunk));
@@ -120,9 +121,9 @@ DownloadMain::close() {
   m_isOpen = false;
 
   m_trackerManager->close();
-  m_delegator.clear();
+  m_delegator.transfer_list()->clear();
 
-  m_content.clear();
+  m_content.bitfield()->unallocate();
   m_content.entry_list()->close();
 
   // Clear the chunklist last as it requires all referenced chunks to
@@ -130,6 +131,8 @@ DownloadMain::close() {
   m_chunkStatistics->clear();
   m_chunkList->clear();
   m_chunkSelector->cleanup();
+
+  m_peerList.clear();
 }
 
 void DownloadMain::start() {
@@ -142,10 +145,9 @@ void DownloadMain::start() {
   m_started = true;
   m_lastConnectedSize = 0;
 
-  // Reset the uploaded baseline every time we restart the download so
-  // that broken trackers get the right uploaded ratio.
-  m_info->set_uploaded_baseline(m_info->up_rate()->total());
-  m_trackerManager->send_start();
+  m_endgame = false;
+  m_delegator.set_aggressive(false);
+  update_endgame();  
 
   receive_connect_peers();
 }  
@@ -165,6 +167,8 @@ DownloadMain::stop() {
   // not to eat available peers.
   m_started = false;
 
+  m_slotStopHandshakes(this);
+
   // Save the addresses we are connected to, so we don't need to
   // perform alot of requests to the tracker when restarting the
   // torrent. Consider saving these in the torrent file when dumping
@@ -196,8 +200,7 @@ DownloadMain::get_bytes_left() const {
 
 void
 DownloadMain::update_endgame() {
-  if (!m_endgame &&
-      m_content.chunks_completed() + m_delegator.get_chunks().size() + 5 >= m_content.chunk_total()) {
+  if (!m_endgame && m_content.chunks_completed() + m_delegator.transfer_list()->size() + 5 >= m_content.chunk_total()) {
     m_endgame = true;
     m_delegator.set_aggressive(true);
   }
@@ -219,12 +222,12 @@ DownloadMain::receive_connect_peers() {
     return;
 
   while (!available_list()->empty() &&
-	 connection_list()->size() < connection_list()->get_min_size() &&
-	 connection_list()->size() + m_slotCountHandshakes(m_info) < connection_list()->get_max_size()) {
+         connection_list()->size() < connection_list()->get_min_size() &&
+         connection_list()->size() + m_slotCountHandshakes(this) < connection_list()->get_max_size()) {
     rak::socket_address sa = available_list()->pop_random();
 
     if (connection_list()->find(sa) == connection_list()->end())
-      m_slotStartHandshake(sa, m_info);
+      m_slotStartHandshake(sa, this);
   }
 }
 
