@@ -42,6 +42,7 @@
 
 #include "available_list.h"
 #include "connection_list.h"
+#include "peer_list.h"
 #include "delegator.h"
 
 #include "data/content.h"
@@ -94,6 +95,7 @@ public:
 
   AvailableList*      available_list()                           { return &m_availableList; }
   ConnectionList*     connection_list()                          { return m_connectionList; }
+  PeerList*           peer_list()                                { return &m_peerList; }
 
   ThrottleList*       upload_throttle()                          { return m_uploadThrottle; }
   void                set_upload_throttle(ThrottleList* t)       { m_uploadThrottle = t; }
@@ -108,11 +110,14 @@ public:
   void                setup_delegator();
   void                setup_tracker();
 
-  typedef rak::mem_fun2<HandshakeManager, void, const rak::socket_address&, DownloadInfo*> SlotStartHandshake;
-  typedef rak::const_mem_fun1<HandshakeManager, uint32_t, DownloadInfo*>                   SlotCountHandshakes;
+  typedef rak::const_mem_fun1<HandshakeManager, uint32_t, DownloadMain*>                   SlotCountHandshakes;
   typedef rak::mem_fun1<DownloadWrapper, void, ChunkHandle>                                SlotHashCheckAdd;
 
-  void                slot_start_handshake(SlotStartHandshake s)   { m_slotStartHandshake = s; }
+  typedef rak::mem_fun2<HandshakeManager, void, const rak::socket_address&, DownloadMain*> slot_start_handshake_type;
+  typedef rak::mem_fun1<HandshakeManager, void, DownloadMain*>                             slot_stop_handshakes_type;
+
+  void                slot_start_handshake(slot_start_handshake_type s) { m_slotStartHandshake = s; }
+  void                slot_stop_handshakes(slot_stop_handshakes_type s) { m_slotStopHandshakes = s; }
   void                slot_count_handshakes(SlotCountHandshakes s) { m_slotCountHandshakes = s; }
   void                slot_hash_check_add(SlotHashCheckAdd s)      { m_slotHashCheckAdd = s; }
 
@@ -146,6 +151,7 @@ private:
 
   AvailableList       m_availableList;
   ConnectionList*     m_connectionList;
+  PeerList            m_peerList;
 
   bool                m_started;
   bool                m_isOpen;
@@ -155,7 +161,9 @@ private:
   ThrottleList*       m_uploadThrottle;
   ThrottleList*       m_downloadThrottle;
 
-  SlotStartHandshake  m_slotStartHandshake;
+  slot_start_handshake_type m_slotStartHandshake;
+  slot_stop_handshakes_type m_slotStopHandshakes;
+
   SlotCountHandshakes m_slotCountHandshakes;
   SlotHashCheckAdd    m_slotHashCheckAdd;
 
