@@ -60,7 +60,7 @@ DownloadManager::erase(DownloadWrapper* d) {
   if (itr == end())
     throw client_error("Tried to remove a torrent that doesn't exist");
     
-  (*itr)->stop();
+  (*itr)->main()->stop();
   (*itr)->close();
 
   delete *itr;
@@ -85,15 +85,15 @@ DownloadManager::find(DownloadInfo* info) {
   return std::find_if(begin(), end(), rak::equal(info, std::mem_fun(&DownloadWrapper::info)));
 }
 
-DownloadInfo*
-DownloadManager::find_info(const std::string& hash) {
+DownloadMain*
+DownloadManager::find_main(const std::string& hash) {
   iterator itr = std::find_if(begin(), end(), rak::equal(hash, rak::on(std::mem_fun(&DownloadWrapper::info), std::mem_fun(&DownloadInfo::hash))));
 
   // TODO: Move these checks somewhere else.
   if (itr == end() || !(*itr)->main()->is_active())
     return NULL;
   else
-    return (*itr)->info();
+    return (*itr)->main();
 }
 
 }
