@@ -73,7 +73,7 @@ DownloadWrapper::DownloadWrapper() :
 
 DownloadWrapper::~DownloadWrapper() {
   if (m_main.is_active())
-    stop();
+    m_main.stop();
 
   if (m_main.is_open())
     close();
@@ -101,134 +101,9 @@ DownloadWrapper::initialize(const std::string& hash, const std::string& id) {
 
   // Connect various signals and slots.
   m_hash->slot_check_chunk(rak::make_mem_fun(this, &DownloadWrapper::check_chunk_hash));
-  m_hash->slot_initial_hash(rak::make_mem_fun(this, &DownloadWrapper::receive_initial_hash));
   m_hash->slot_storage_error(rak::make_mem_fun(this, &DownloadWrapper::receive_storage_error));
-}
 
-void
-DownloadWrapper::hash_resume_load() {
-  if (!m_main.is_open() || m_main.is_active() || m_hash->is_checked())
-    throw client_error("DownloadWrapper::resume_load() called with wrong state");
-
-  if (!m_bencode->has_key_map("libtorrent resume"))
-    return;
-
-  try {
-    Object& resume  = m_bencode->get_key("libtorrent resume");
-
-    // Load peer addresses.
-    if (resume.has_key_string("peers"))
-      insert_available_list(resume.get_key("peers").as_string());
-
-    Object& files = resume.get_key("files");
-
-    if (resume.has_key_string("bitfield") &&
-	resume.get_key("bitfield").as_string().size() == m_main.content()->bitfield()->size_bytes() &&
-	files.as_list().size() == m_main.content()->entry_list()->files_size()) {
-
-      // Clear the hash checking ranges, and add the files ranges we
-      // must check.
-      m_hash->ranges().clear();
-      m_main.content()->bitfield()->from_c_str(resume.get_key("bitfield").as_string().c_str());
-
-    } else {
-      // No resume bitfield available, check the whole range.
-      m_hash->ranges().insert(0, m_main.content()->chunk_total());
-    }
-
-    Object::list_type::iterator bItr = files.as_list().begin();
-    EntryList::iterator sItr = m_main.content()->entry_list()->begin();
-
-    // Check the validity of each file, add to the m_hash's ranges if invalid.
-    while (sItr != m_main.content()->entry_list()->end()) {
-      rak::file_stat fs;
-
-      // Check that the size and modified stamp matches. If not, then
-      // add to the hashes to check.
-
-      if (!fs.update((*sItr)->file_meta()->get_path()) ||
-	  (*sItr)->size() != fs.size() ||
-	  !bItr->has_key_value("mtime") ||
-	  bItr->get_key("mtime").as_value() != fs.modified_time())
-	m_hash->ranges().insert((*sItr)->range().first, (*sItr)->range().second);
-
-      // Update the priority from the fast resume data.
-      if (bItr->has_key_value("priority") &&
-	  bItr->get_key("priority").as_value() >= 0 &&
-	  bItr->get_key("priority").as_value() < 3)
-	(*sItr)->set_priority((*bItr).get_key("priority").as_value());
-
-      ++sItr;
-      ++bItr;
-    }  
-
-  } catch (bencode_error e) {
-    m_hash->ranges().insert(0, m_main.content()->chunk_total());
-  }
-
-  // Clear bits in invalid regions which will be checked by m_hash.
-  //
-  // If this is ever optimized in such a way as not to update the
-  // bitfield's set size, make sure that Content::update_done() calls
-  // Bitfield::update().
-  for (HashTorrent::Ranges::iterator itr = m_hash->ranges().begin(); itr != m_hash->ranges().end(); ++itr)
-    m_main.content()->bitfield()->unset_range(itr->first, itr->second);
-
-  m_main.content()->update_done();
-}
-
-// Break this function up into several smaller functions to make it
-// easier to read.
-void
-DownloadWrapper::hash_resume_save() {
-  if (!m_main.is_open() || m_main.is_active())
-    throw client_error("DownloadWrapper::resume_save() called with wrong state");
-
-  if (!m_hash->is_checked())
-    // We don't remove the old hash data since it might still be
-    // valid, just that the client didn't finish the check this time.
-    return;
-
-  // If we can't sync, don't save the resume data.
-  if (m_main.chunk_list()->sync_all(MemoryChunk::sync_sync) != 0 && m_bencode->has_key("libtorrent resume")) {
-    m_bencode->get_key("libtorrent resume").erase_key("bitfield");
-    return;
-  }
-
-  // Clear the resume data since if the syncing fails we propably don't
-  // want the old resume data.
-  Object& resume = m_bencode->insert_key("libtorrent resume", Object(Object::TYPE_MAP));
-
-  resume.insert_key("bitfield", std::string((char*)m_main.content()->bitfield()->begin(), m_main.content()->bitfield()->size_bytes()));
-
-  Object::list_type& l = resume.insert_key("files", Object(Object::TYPE_LIST)).as_list();
-
-  EntryList::iterator sItr = m_main.content()->entry_list()->begin();
-  
-  // Check the validity of each file, add to the m_hash's ranges if invalid.
-  while (sItr != m_main.content()->entry_list()->end()) {
-    Object& b = *l.insert(l.end(), Object(Object::TYPE_MAP));
-
-    rak::file_stat fs;
-
-    if (!fs.update((*sItr)->file_meta()->get_path())) {
-      l.clear();
-      break;
-    }
-
-    b.insert_key("mtime", fs.modified_time());
-    b.insert_key("priority", (int)(*sItr)->priority());
-
-    ++sItr;
-  }
-
-  // Save the available peer list. Since this function is called when
-  // the download is stopped, we know that all the previously
-  // connected peers have been copied to the available list.
-  //
-  // Consider whetever we need to add currently connected too, as we
-  // might save the session while still active.
-  extract_available_list(&resume.insert_key("peers", Object()));
+  m_hash->delay_checked().set_slot(rak::mem_fn(this, &DownloadWrapper::receive_initial_hash));
 }
 
 void
@@ -248,7 +123,7 @@ DownloadWrapper::close() {
 
   // Clear after m_hash to ensure that the empty hash done signal does
   // not get passed to HashTorrent.
-  m_hash->get_queue()->remove(info()->hash());
+  m_hash->get_queue()->remove(this);
 
   // This could/should be async as we do not care that much if it
   // succeeds or not, any chunks not included in that last
@@ -261,37 +136,9 @@ DownloadWrapper::close() {
   priority_queue_erase(&taskScheduler, &m_delayDownloadDone);
 }
 
-void
-DownloadWrapper::start() {
-  if (!m_hash->is_checked())
-    throw client_error("Tried to start an unchecked download");
-
-  if (!m_main.is_open())
-    throw client_error("Tried to start a closed download");
-
-  if (m_main.is_active())
-    return;
-
-  m_connectionChunkPassed = signal_chunk_passed().connect(sigc::mem_fun(m_main.delegator(), &Delegator::done));
-  m_connectionChunkFailed = signal_chunk_failed().connect(sigc::mem_fun(m_main.delegator(), &Delegator::redo));
-
-  m_main.start();
-}
-
-// Remember to update receive_tick() when changing stop().
-void
-DownloadWrapper::stop() {
-  m_main.stop();
-
-  m_connectionChunkPassed.disconnect();
-  m_connectionChunkFailed.disconnect();
-
-  m_main.tracker_manager()->send_stop();
-}
-
 bool
 DownloadWrapper::is_stopped() const {
-  return !m_main.tracker_manager()->is_active();
+  return !m_main.tracker_manager()->is_active() && !m_main.tracker_manager()->is_busy();
 }
 
 void
@@ -299,25 +146,22 @@ DownloadWrapper::insert_available_list(const std::string& src) {
   std::list<rak::socket_address> l;
 
   std::copy(reinterpret_cast<const SocketAddressCompact*>(src.c_str()),
-	    reinterpret_cast<const SocketAddressCompact*>(src.c_str() + src.size() - src.size() % sizeof(SocketAddressCompact)),
-	    std::back_inserter(l));
+            reinterpret_cast<const SocketAddressCompact*>(src.c_str() + src.size() - src.size() % sizeof(SocketAddressCompact)),
+            std::back_inserter(l));
   l.sort();
 
   m_main.available_list()->insert(&l);
 }
 
 void
-DownloadWrapper::extract_available_list(Object* dest) {
-  *dest = std::string();
-  
-  std::string& peers = dest->as_string();
-  peers.reserve(m_main.available_list()->size() * sizeof(SocketAddressCompact));
+DownloadWrapper::extract_available_list(std::string& dest) {
+  dest.reserve(m_main.available_list()->size() * sizeof(SocketAddressCompact));
   
   for (AvailableList::const_iterator itr = m_main.available_list()->begin(), last = m_main.available_list()->end(); itr != last; ++itr) {
     if (itr->family() == rak::socket_address::af_inet) {
       SocketAddressCompact sac(itr->sa_inet()->address_n(), itr->sa_inet()->port_n());
 
-      peers.append(sac.c_str(), sizeof(SocketAddressCompact));
+      dest.append(sac.c_str(), sizeof(SocketAddressCompact));
     }
   }  
 }
@@ -336,22 +180,25 @@ DownloadWrapper::receive_initial_hash() {
   if (m_main.is_active())
     throw internal_error("DownloadWrapper::receive_initial_hash() but we're in a bad state.");
 
-  if (!m_hash->is_checked()) {
+  if (!m_hash->is_checking()) {
     m_hash->clear();
 
     // Clear after m_hash to ensure that the empty hash done signal does
     // not get passed to HashTorrent.
-    m_hash->get_queue()->remove(info()->hash());
-    m_main.content()->clear();
+    m_hash->get_queue()->remove(this);
+    m_main.content()->bitfield()->unset_all();
 
-  } else if (!m_main.content()->entry_list()->resize_all()) {
+  } else if (m_main.content()->entry_list()->resize_all()) {
+    m_hash->confirm_checked();
+
+  } else {
     // We couldn't resize the files, tell the client.
     receive_storage_error("Could not resize files in the torrent.");
 
     // Do we clear the hash?
   }
 
-  if (m_hash->get_queue()->has(info()->hash()))
+  if (m_hash->get_queue()->has(this))
     throw internal_error("DownloadWrapper::receive_initial_hash() found a chunk in the HashQueue.");
 
   // Initialize the ChunkSelector here so that no chunks will be
@@ -365,51 +212,56 @@ DownloadWrapper::receive_initial_hash() {
 void
 DownloadWrapper::check_chunk_hash(ChunkHandle handle) {
   // Using HashTorrent's queue temporarily.
-  m_hash->get_queue()->push_back(handle, rak::make_mem_fun(this, &DownloadWrapper::receive_hash_done), info()->hash());
+  m_hash->get_queue()->push_back(handle, rak::make_mem_fun(this, &DownloadWrapper::receive_hash_done));
 }
 
 void
-DownloadWrapper::receive_hash_done(ChunkHandle handle, std::string h) {
+DownloadWrapper::receive_hash_done(ChunkHandle handle, const char* hash) {
   if (!handle.is_valid())
     throw internal_error("DownloadWrapper::receive_hash_done(...) called on an invalid chunk.");
 
   if (!is_open())
     throw internal_error("DownloadWrapper::receive_hash_done(...) called but the download is not open.");
 
-  uint32_t index = handle->index();
-  m_main.chunk_list()->release(handle);
+  if (hash == NULL) {
+    // Clearing the hash queue, do nothing. Add checks here?
 
-  if (m_hash->is_checking()) {
-    m_main.content()->receive_chunk_hash(index, h);
+  } else if (m_hash->is_checking()) {
+    m_main.content()->receive_chunk_hash(handle.index(), hash);
     m_hash->receive_chunkdone();
 
   } else if (m_hash->is_checked()) {
+    // Receiving chunk hashes after stopping the torrent should be
+    // safe.
 
-    if (m_main.chunk_selector()->bitfield()->get(index))
+    if (m_main.chunk_selector()->bitfield()->get(handle.index()))
       throw internal_error("DownloadWrapper::receive_hash_done(...) received a chunk that isn't set in ChunkSelector.");
 
-    if (m_main.content()->receive_chunk_hash(index, h)) {
+    if (m_main.content()->receive_chunk_hash(handle.index(), hash)) {
 
-      // Should this be done here, or after send_finished_chunk?
-      signal_chunk_passed().emit(index);
+      m_main.delegator()->transfer_list()->hash_succeded(handle.index());
       m_main.update_endgame();
 
       if (m_main.content()->is_done())
-	finished_download();
+        finished_download();
     
-      m_main.connection_list()->send_finished_chunk(index);
+      m_main.connection_list()->send_finished_chunk(handle.index());
+      signal_chunk_passed().emit(handle.index());
 
     } else {
-      signal_chunk_failed().emit(index);
+      // This needs to ensure the chunk is still valid.
+      m_main.delegator()->transfer_list()->hash_failed(handle.index(), handle.chunk());
+      signal_chunk_failed().emit(handle.index());
     }
 
   } else {
-    // When the HashQueue gets cleared, it will trigger this signal to
-    // do cleanup. As HashTorrent must be cleared before the
-    // HashQueue, we need to ignore the chunks that get triggered.
+    // When clearing, we get a NULL string, so if none of the above
+    // caught this hash, then we got a problem.
 
-    //throw internal_error("DownloadWrapper::receive_hash_done(...) called but we're not in the right state.");
+    throw internal_error("DownloadWrapper::receive_hash_done(...) Was not expecting non-NULL hash.");
   }
+
+  m_main.chunk_list()->release(&handle);
 }  
 
 void
@@ -480,8 +332,7 @@ DownloadWrapper::receive_update_priorities() {
 
   m_main.chunk_selector()->update_priorities();
 
-  std::for_each(m_main.connection_list()->begin(), m_main.connection_list()->end(),
-		std::mem_fun(&PeerConnectionBase::update_interested));
+  std::for_each(m_main.connection_list()->begin(), m_main.connection_list()->end(), std::mem_fun(&PeerConnectionBase::update_interested));
 }
 
 void
