@@ -73,16 +73,9 @@ public:
   // Initialize hash checker and various download stuff.
   void                initialize(const std::string& hash, const std::string& id);
 
-  // Don't load unless the object is newly initialized.
-  void                hash_resume_load();
-  void                hash_resume_save();
-
   void                open();
   void                close();
 
-  void                start();
-  void                stop();
-
   bool                is_open() const                         { return m_main.is_open(); }
   bool                is_stopped() const;
 
@@ -99,13 +92,13 @@ public:
   void                set_connection_type(int t)              { m_connectionType = t; }
 
   void                insert_available_list(const std::string& src);
-  void                extract_available_list(Object* dest);
+  void                extract_available_list(std::string& dest);
 
   void                receive_keepalive();
   void                receive_initial_hash();
 
   void                check_chunk_hash(ChunkHandle handle);
-  void                receive_hash_done(ChunkHandle handle, std::string h);
+  void                receive_hash_done(ChunkHandle handle, const char* hash);
 
   void                receive_storage_error(const std::string& str);
   void                receive_tracker_success(AddressList* l);
