@@ -78,9 +78,7 @@ Manager::Manager() :
 
   priority_queue_insert(&taskScheduler, &m_taskTick, cachedTime.round_seconds());
 
-  m_handshakeManager->slot_connected(rak::make_mem_fun(this, &Manager::receive_connection));
-  m_handshakeManager->slot_download_id(rak::make_mem_fun(m_downloadManager, &DownloadManager::find_info));
-
+  m_handshakeManager->slot_download_id(rak::make_mem_fun(m_downloadManager, &DownloadManager::find_main));
   m_connectionManager->listen()->slot_incoming(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::add_incoming));
 }
 
@@ -105,6 +103,7 @@ void
 Manager::initialize_download(DownloadWrapper* d) {
   d->main()->slot_count_handshakes(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::size_info));
   d->main()->slot_start_handshake(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::add_outgoing));
+  d->main()->slot_stop_handshakes(rak::make_mem_fun(m_handshakeManager, &HandshakeManager::erase_download));
 
   d->hash_checker()->set_queue(m_hashQueue);
 
@@ -124,8 +123,6 @@ Manager::initialize_download(DownloadWrapper* d) {
 
 void
 Manager::cleanup_download(DownloadWrapper* d) {
-  m_handshakeManager->erase_info(d->info());
-
   m_resourceManager->erase(d->main());
   m_downloadManager->erase(d);
 }
@@ -146,26 +143,4 @@ Manager::receive_tick() {
   priority_queue_insert(&taskScheduler, &m_taskTick, (cachedTime + rak::timer::from_seconds(30)).round_seconds());
 }
 
-void
-Manager::receive_connection(SocketFd fd, DownloadInfo* info, const PeerInfo& peer) {
-  DownloadManager::iterator itr = m_downloadManager->find(info);
-  
-  if (itr == m_downloadManager->end() ||
-      peer.get_id() == info->local_id() ||
-      !peer.socket_address()->is_valid()) {
-
-    m_connectionManager->dec_socket_count();
-    fd.close();
-    
-    return;
-  }
-
-  if (!(*itr)->main()->is_active() ||
-      !(*itr)->main()->connection_list()->insert((*itr)->main(), peer, fd)) {
-    m_connectionManager->dec_socket_count();
-    fd.close();
-    return;
-  }
-}
-
 }
