@@ -41,12 +41,8 @@
 #include <cerrno>
 #include <cstdio>
 #include <cstring>
-#include <sys/types.h>
-#include <sys/socket.h>
 #include <rak/error_number.h>
 
-#include "torrent/exceptions.h"
-
 namespace torrent {
 
 std::string
@@ -58,22 +54,6 @@ int_to_string(int v) {
   return buf;
 }
 
-inline int
-SocketStream::read_stream(void* buf, uint32_t length) {
-  if (length == 0)
-    throw internal_error("Tried to read buffer length 0");
-
-  return ::recv(m_fileDesc, buf, length, 0);
-}
-
-inline int
-SocketStream::write_stream(const void* buf, uint32_t length) {
-  if (length == 0)
-    throw internal_error("Tried to write buffer length 0");
-
-  return ::send(m_fileDesc, buf, length, 0);
-}
-
 uint32_t
 SocketStream::read_stream_throws(void* buf, uint32_t length) {
   int r = read_stream(buf, length);
