@@ -146,12 +146,14 @@ ThrottleList::update_quota(uint32_t quota) {
 uint32_t
 ThrottleList::node_quota(ThrottleNode* node) {
   if (!m_enabled) {
-    return std::numeric_limits<uint32_t>::max();
+    // Returns max for signed integer to ensure we don't overflow
+    // claculations.
+    return std::numeric_limits<int32_t>::max();
 
   } else if (!is_active(node)) {
     throw internal_error(is_inactive(node) ?
-			 "ThrottleList::node_quota(...) called on an inactive node." :
-			 "ThrottleList::node_quota(...) could not find node.");
+                         "ThrottleList::node_quota(...) called on an inactive node." :
+                         "ThrottleList::node_quota(...) could not find node.");
 
   } else if (node->quota() + m_unallocatedQuota >= m_minChunkSize) {
     return node->quota() + m_unallocatedQuota;
@@ -183,8 +185,8 @@ void
 ThrottleList::node_deactivate(ThrottleNode* node) {
   if (!is_active(node))
     throw internal_error(is_inactive(node) ?
-			 "ThrottleList::node_deactivate(...) called on an inactive node." :
-			 "ThrottleList::node_deactivate(...) could not find node.");
+                         "ThrottleList::node_deactivate(...) called on an inactive node." :
+                         "ThrottleList::node_deactivate(...) could not find node.");
 
   Base::splice(end(), *this, node->list_iterator());
 
