@@ -43,8 +43,10 @@
 
 #include "torrent/exceptions.h"
 #include "download/download_info.h"
+#include "download/download_main.h"
 
 #include "peer_info.h"
+#include "peer_connection_base.h"
 #include "handshake.h"
 #include "handshake_manager.h"
 
@@ -62,8 +64,8 @@ HandshakeManager::delete_handshake(Handshake* h) {
 }
 
 HandshakeManager::size_type
-HandshakeManager::size_info(DownloadInfo* info) const {
-  return std::count_if(Base::begin(), Base::end(), rak::equal(info, std::mem_fun(&Handshake::download_info)));
+HandshakeManager::size_info(DownloadMain* info) const {
+  return std::count_if(Base::begin(), Base::end(), rak::equal(info, std::mem_fun(&Handshake::download)));
 }
 
 void
@@ -84,7 +86,7 @@ HandshakeManager::erase(Handshake* handshake) {
 
 struct handshake_manager_equal : std::binary_function<const rak::socket_address*, const Handshake*, bool> {
   bool operator () (const rak::socket_address* sa1, const Handshake* p2) const {
-    return *sa1 == *p2->peer_info()->socket_address();
+    return p2->peer_info() != NULL && *sa1 == *p2->peer_info()->socket_address();
   }
 };
 
@@ -94,8 +96,8 @@ HandshakeManager::find(const rak::socket_address& sa) {
 }
 
 void
-HandshakeManager::erase_info(DownloadInfo* info) {
-  iterator split = std::partition(Base::begin(), Base::end(), rak::not_equal(info, std::mem_fun(&Handshake::download_info)));
+HandshakeManager::erase_download(DownloadMain* info) {
+  iterator split = std::partition(Base::begin(), Base::end(), rak::not_equal(info, std::mem_fun(&Handshake::download)));
 
   std::for_each(split, Base::end(), std::bind1st(std::mem_fun(&HandshakeManager::delete_handshake), this));
   Base::erase(split, Base::end());
@@ -119,7 +121,7 @@ HandshakeManager::add_incoming(SocketFd fd, const rak::socket_address& sa) {
 }
   
 void
-HandshakeManager::add_outgoing(const rak::socket_address& sa, DownloadInfo* info) {
+HandshakeManager::add_outgoing(const rak::socket_address& sa, DownloadMain* info) {
   if (!manager->connection_manager()->can_connect() ||
       !manager->connection_manager()->filter(sa.c_sockaddr()))
     return;
@@ -148,24 +150,55 @@ HandshakeManager::add_outgoing(const rak::socket_address& sa, DownloadInfo* info
 
 void
 HandshakeManager::receive_succeeded(Handshake* h) {
+  if (!h->is_active())
+    throw internal_error("HandshakeManager::receive_succeeded(...) called on an inactive handshake.");
+
   erase(h);
   h->clear();
 
-//   h->download_info()->signal_network_log().emit("Successful handshake: " + h->peer_info()->get_address());
+  PeerConnectionBase* pcb;
 
-  m_slotConnected(h->get_fd(), h->download_info(), *h->peer_info());
+  if (h->download()->is_active() &&
 
-  h->set_fd(SocketFd());
+      // We need to make libtorrent more selective in the clients it
+      // connects to, and to move this somewhere else.
+      (!h->download()->content()->is_done() || !h->bitfield()->is_all_set()) &&
+
+      (pcb = h->download()->connection_list()->insert(h->download(), h->peer_info(), h->get_fd(), h->bitfield())) != NULL) {
+
+//     h->download()->info()->signal_network_log().emit("Successful handshake: " + h->peer_info()->socket_address()->address_str());
+    h->set_peer_info(NULL);
+
+    post_insert(h, pcb);
+
+  } else {
+    manager->connection_manager()->dec_socket_count();
+    h->get_fd().close();
+
+//     h->download()->info()->signal_network_log().emit("Successful handshake, failed: " + h->peer_info()->socket_address()->address_str());
+  }
+
+  h->get_fd().clear();
   delete h;
 }
 
+inline void
+HandshakeManager::post_insert(Handshake* h, PeerConnectionBase* pcb) {
+  if (h->unread_size() != 0) {
+    pcb->push_unread(h->unread_data(), h->unread_size());
+    pcb->event_read();
+  }
+}
+
 void
 HandshakeManager::receive_failed(Handshake* h) {
+  if (!h->is_active())
+    throw internal_error("HandshakeManager::receive_failed(...) called on an inactive handshake.");
+
+//   if (h->download() != NULL)
+//     h->download()->info()->signal_network_log().emit("Failed handshake: " + h->socket_address()->address_str());
+
   erase(h);
-
-//   if (h->download_info() != NULL)
-//     h->download_info()->signal_network_log().emit("Failed handshake: " + h->peer_info()->get_address());
-
   delete_handshake(h);
 }
 
