@@ -40,6 +40,8 @@
 #include <rak/error_number.h>
 
 #include "torrent/exceptions.h"
+#include "torrent/block.h"
+#include "data/chunk_iterator.h"
 #include "data/chunk_list.h"
 #include "download/choke_manager.h"
 #include "download/chunk_selector.h"
@@ -60,6 +62,8 @@ PeerConnectionBase::PeerConnectionBase() :
   m_down(new ProtocolRead()),
   m_up(new ProtocolWrite()),
 
+  m_peerInfo(NULL),
+
   m_downStall(0),
 
   m_sendChoked(false),
@@ -73,6 +77,11 @@ PeerConnectionBase::~PeerConnectionBase() {
   if (m_download == NULL)
     throw internal_error("PeerConnection::~PeerConnection() m_fd is valid but m_state and/or m_net is NULL");
 
+  m_downloadQueue.clear();
+
+  up_chunk_release();
+  down_chunk_release();
+
   m_download->choke_manager()->disconnected(this);
   m_download->chunk_statistics()->received_disconnect(&m_peerChunks);
 
@@ -86,13 +95,9 @@ PeerConnectionBase::~PeerConnectionBase() {
   get_fd().close();
   get_fd().clear();
 
-  if (download_queue()->is_downloading())
-    download_queue()->skip();
-
-  up_chunk_release();
-  down_chunk_release();
-
-  download_queue()->cancel();
+  // Need to move more stuff into download*.
+  m_download->peer_list()->disconnected(m_peerInfo);
+  m_peerInfo = NULL;
 
   m_download->upload_throttle()->erase(m_peerChunks.upload_throttle());
   m_download->download_throttle()->erase(m_peerChunks.download_throttle());
@@ -107,15 +112,21 @@ PeerConnectionBase::~PeerConnectionBase() {
 }
 
 void
-PeerConnectionBase::initialize(DownloadMain* download, const PeerInfo& p, SocketFd fd) {
+PeerConnectionBase::initialize(DownloadMain* download, PeerInfo* peerInfo, SocketFd fd, Bitfield* bitfield) {
   if (get_fd().is_valid())
-    throw internal_error("Tried to re-set PeerConnection");
+    throw internal_error("Tried to re-set PeerConnection.");
+
+  if (!peerInfo->is_valid() || !fd.is_valid())
+    throw internal_error("PeerConnectionBase::set(...) received bad input.");
 
   set_fd(fd);
 
-  m_peer = p;
+  m_peerInfo = peerInfo;
   m_download = download;
 
+  m_peerChunks.set_peer_info(m_peerInfo);
+  m_peerChunks.bitfield()->swap(*bitfield);
+
   m_peerChunks.upload_throttle()->set_list_iterator(m_download->upload_throttle()->end());
   m_peerChunks.upload_throttle()->slot_activate(rak::make_mem_fun(this, &PeerConnectionBase::receive_throttle_up_activate));
 
@@ -125,11 +136,6 @@ PeerConnectionBase::initialize(DownloadMain* download, const PeerInfo& p, Socket
   download_queue()->set_delegator(m_download->delegator());
   download_queue()->set_peer_chunks(&m_peerChunks);
 
-  if (m_download == NULL || !p.is_valid() || !get_fd().is_valid())
-    throw internal_error("PeerConnectionBase::set(...) recived bad input.");
-
-  m_peerChunks.bitfield()->resize(m_download->content()->chunk_total());
-
   manager->poll()->open(this);
   manager->poll()->insert_read(this);
   manager->poll()->insert_write(this);
@@ -137,30 +143,17 @@ PeerConnectionBase::initialize(DownloadMain* download, const PeerInfo& p, Socket
 
   m_timeLastRead = cachedTime;
 
+  m_download->chunk_statistics()->received_connect(&m_peerChunks);
+
+  // Hmm... cleanup?
+  update_interested();
+
   initialize_custom();
 }
 
-void
-PeerConnectionBase::load_down_chunk(const Piece& p) {
-  m_downPiece = p;
-
-  if (!m_download->content()->is_valid_piece(p))
-    throw internal_error("Incoming pieces list contains a bad piece");
-  
-  if (m_downChunk.is_valid() && p.index() == m_downChunk->index())
-    return;
-
-  down_chunk_release();
-
-  m_downChunk = m_download->chunk_list()->get(p.index(), true);
-  
-  if (!m_downChunk.is_valid())
-    throw storage_error("File chunk write error: " + std::string(m_downChunk.error_number().c_str()));
-}
-
 void
 PeerConnectionBase::load_up_chunk() {
-  if (m_upChunk.is_valid() && m_upChunk->index() == m_upPiece.index())
+  if (m_upChunk.is_valid() && m_upChunk.index() == m_upPiece.index())
     return;
 
   up_chunk_release();
@@ -216,12 +209,59 @@ PeerConnectionBase::event_error() {
   m_download->connection_list()->erase(this);
 }
 
+bool
+PeerConnectionBase::down_chunk_start(const Piece& piece) {
+  if (!download_queue()->downloading(piece)) {
+    if (piece.length() == 0)
+      m_download->info()->signal_network_log().emit("Received piece with length zero.");
+
+    return false;
+  }
+
+  if (!m_download->content()->is_valid_piece(piece))
+    throw internal_error("Incoming pieces list contains a bad piece.");
+  
+  if (!m_downChunk.is_valid() || piece.index() != m_downChunk.index()) {
+    down_chunk_release();
+    m_downChunk = m_download->chunk_list()->get(piece.index(), true);
+  
+    if (!m_downChunk.is_valid())
+      throw storage_error("File chunk write error: " + std::string(m_downChunk.error_number().c_str()) + ".");
+  }
+
+  return m_downloadQueue.transfer()->is_leader();
+}
+
+void
+PeerConnectionBase::down_chunk_finished() {
+  if (!download_queue()->transfer()->is_finished())
+    throw internal_error("PeerConnectionBase::down_chunk_finished() Transfer not finished.");
+
+  if (download_queue()->transfer()->is_leader()) {
+    if (!m_downChunk.is_valid())
+      throw internal_error("PeerConnectionBase::down_chunk_finished() Transfer is the leader, but no chunk allocated.");
+
+    download_queue()->finished();
+    m_downChunk.object()->set_time_modified(cachedTime);
+
+  } else {
+    download_queue()->skipped();
+  }
+        
+  if (m_downStall > 0)
+    m_downStall--;
+        
+  // TODO: clear m_down.data?
+  // TODO: remove throttle if choked? Rarely happens though.
+  write_insert_poll_safe();
+}
+
 bool
 PeerConnectionBase::down_chunk() {
   if (!m_download->download_throttle()->is_throttled(m_peerChunks.download_throttle()))
     throw internal_error("PeerConnectionBase::down_chunk() tried to read a piece but is not in throttle list");
 
-  if (!m_downChunk->chunk()->is_writable())
+  if (!m_downChunk.chunk()->is_writable())
     throw internal_error("PeerConnectionBase::down_part() chunk not writable, permission denided");
 
   uint32_t quota = m_download->download_throttle()->node_quota(m_peerChunks.download_throttle());
@@ -232,56 +272,138 @@ PeerConnectionBase::down_chunk() {
     return false;
   }
 
-  uint32_t count;
-  uint32_t left = quota = std::min(quota, m_downPiece.length() - m_down->position());
+  uint32_t bytesTransfered = 0;
+  BlockTransfer* transfer = m_downloadQueue.transfer();
 
-  Chunk::MemoryArea memory;
-  ChunkPart part = m_downChunk->chunk()->at_position(m_downPiece.offset() + m_down->position());
+  Chunk::data_type data;
+  ChunkIterator itr(m_downChunk.chunk(),
+                    transfer->piece().offset() + transfer->position(),
+                    transfer->piece().offset() + std::min(transfer->position() + quota, transfer->piece().length()));
 
   do {
-    memory = m_downChunk->chunk()->at_memory(m_downPiece.offset() + m_down->position(), part++);
-    count = read_stream_throws(memory.first, std::min(left, memory.second));
+    data = itr.data();
+    data.second = read_stream_throws(data.first, data.second);
 
-    m_down->adjust_position(count);
-    left -= count;
+    bytesTransfered += data.second;
 
-  } while (count == memory.second && left != 0);
+  } while (itr.used(data.second));
 
-  uint32_t bytes = quota - left;
+  transfer->adjust_position(bytesTransfered);
 
-  m_download->download_throttle()->node_used(m_peerChunks.download_throttle(), bytes);
-  m_download->info()->down_rate()->insert(bytes);
+  m_download->download_throttle()->node_used(m_peerChunks.download_throttle(), bytesTransfered);
+  m_download->info()->down_rate()->insert(bytesTransfered);
 
-  return m_down->position() == m_downPiece.length();
+  return transfer->is_finished();
 }
 
 bool
 PeerConnectionBase::down_chunk_from_buffer() {
-  uint32_t count, quota;
-  uint32_t left = quota = std::min<uint32_t>(m_down->buffer()->remaining(),
-					     m_downPiece.length() - m_down->position());
+  m_down->buffer()->move_position(down_chunk_process(m_down->buffer()->position(), m_down->buffer()->remaining()));
 
-  Chunk::MemoryArea memory;
-  ChunkPart part = m_downChunk->chunk()->at_position(m_downPiece.offset() + m_down->position());
+  if (m_downloadQueue.transfer()->is_finished() && m_down->buffer()->remaining() != 0)
+    throw internal_error("PeerConnectionBase::down_chunk_from_buffer() transfer->is_finished() && m_down->buffer()->remaining() != 0.");
 
-  do {
-    memory = m_downChunk->chunk()->at_memory(m_downPiece.offset() + m_down->position(), part++);
-    count = std::min(left, memory.second);
+  return m_downloadQueue.transfer()->is_finished();
+}  
 
-    std::memcpy(memory.first, m_down->buffer()->position(), count);
+// When this transfer again becomes the leader, we just return false
+// and wait for the next polling. It is an exceptional case so we
+// don't really care that much about performance.
+bool
+PeerConnectionBase::down_chunk_skip() {
+  uint32_t length = read_stream_throws(m_nullBuffer, m_downloadQueue.transfer()->piece().length() - m_downloadQueue.transfer()->position());
 
-    m_down->adjust_position(count);
-    m_down->buffer()->move_position(count);
-    left -= count;
+  if (down_chunk_skip_process(m_nullBuffer, length) != length)
+    throw internal_error("PeerConnectionBase::down_chunk_skip() down_chunk_skip_process(m_nullBuffer, length) != length.");
 
-  } while (left != 0);
+  return m_downloadQueue.transfer()->is_finished();
+}
 
-  uint32_t bytes = quota - left;
+bool
+PeerConnectionBase::down_chunk_skip_from_buffer() {
+  m_down->buffer()->move_position(down_chunk_skip_process(m_down->buffer()->position(), m_down->buffer()->remaining()));
+  
+  return m_downloadQueue.transfer()->is_finished();
+}
 
-  m_download->download_throttle()->node_used(m_peerChunks.download_throttle(), bytes);
-  m_download->info()->down_rate()->insert(bytes);
+// Process data from a leading transfer.
+uint32_t
+PeerConnectionBase::down_chunk_process(const void* buffer, uint32_t length) {
+  if (!m_downChunk.is_valid() || m_downChunk.index() != m_downloadQueue.transfer()->index())
+    throw internal_error("PeerConnectionBase::down_chunk_process(...) !m_downChunk.is_valid() || m_downChunk.index() != m_downloadQueue.transfer()->index().");
 
-  return m_down->position() == m_downPiece.length();
+  if (length == 0)
+    return length;
+
+  BlockTransfer* transfer = m_downloadQueue.transfer();
+
+  length = std::min(transfer->piece().length() - transfer->position(), length);
+
+  m_downChunk.chunk()->from_buffer(buffer, transfer->piece().offset() + transfer->position(), length);
+
+  transfer->adjust_position(length);
+
+  m_download->download_throttle()->node_used(m_peerChunks.download_throttle(), length);
+  m_download->info()->down_rate()->insert(length);
+
+  return length;
+}
+
+// Process data from non-leading transfer. If this transfer encounters
+// mismatching data with the leader then bork this transfer. If we get
+// ahead of the leader, we switch the leader.
+uint32_t
+PeerConnectionBase::down_chunk_skip_process(const void* buffer, uint32_t length) {
+  BlockTransfer* transfer = m_downloadQueue.transfer();
+
+  // Adjust 'length' to be less than or equal to what is remaining of
+  // the block to simplify the rest of the function.
+  length = std::min(length, transfer->piece().length() - transfer->position());
+
+  m_download->download_throttle()->node_used(m_peerChunks.download_throttle(), length);
+  m_download->info()->down_rate()->insert(length);
+
+  if (!transfer->is_valid()) {
+    transfer->adjust_position(length);
+    return length;
+  }
+
+  if (!transfer->block()->is_transfering())
+    throw internal_error("PeerConnectionBase::down_chunk_skip_process(...) block is not transfering, yet we have non-leaders.");
+
+  // Temporary test.
+  if (transfer->position() > transfer->block()->leader()->position())
+    throw internal_error("PeerConnectionBase::down_chunk_skip_process(...) transfer is past the Block's position.");
+
+  // If the transfer is valid, compare the downloaded data to the
+  // leader.
+  uint32_t compareLength = std::min(length, transfer->block()->leader()->position() - transfer->position());
+
+  // The data doesn't match with what has previously been downloaded,
+  // bork this transfer.
+  if (!m_downChunk.chunk()->compare_buffer(buffer, transfer->piece().offset() + transfer->position(), compareLength)) {
+    m_download->info()->signal_network_log().emit("Data does not match what was previously downloaded.");
+    
+    m_downloadQueue.transfer_dissimilar();
+    m_downloadQueue.transfer()->adjust_position(length);
+
+    return length;
+  }
+
+  transfer->adjust_position(compareLength);
+
+  if (compareLength == length)
+    return length;
+
+  // Add another check here to see if we really want to be the new
+  // leader.
+
+  transfer->block()->change_leader(transfer);
+
+  if (down_chunk_process(static_cast<const char*>(buffer) + compareLength, length - compareLength) != length - compareLength)
+    throw internal_error("PeerConnectionBase::down_chunk_skip_process(...) down_chunk_process(...) returned wrong value.");
+  
+  return length;
 }
 
 bool
@@ -289,7 +411,7 @@ PeerConnectionBase::up_chunk() {
   if (!m_download->upload_throttle()->is_throttled(m_peerChunks.upload_throttle()))
     throw internal_error("PeerConnectionBase::up_chunk() tried to write a piece but is not in throttle list");
 
-  if (!m_upChunk->chunk()->is_readable())
+  if (!m_upChunk.chunk()->is_readable())
     throw internal_error("ProtocolChunk::write_part() chunk not readable, permission denided");
 
   uint32_t quota = m_download->upload_throttle()->node_quota(m_peerChunks.upload_throttle());
@@ -300,43 +422,40 @@ PeerConnectionBase::up_chunk() {
     return false;
   }
 
-  uint32_t count;
-  uint32_t left = quota = std::min(quota, m_upPiece.length() - m_up->position());
+  uint32_t bytesTransfered = 0;
 
-  Chunk::MemoryArea memory;
-  ChunkPart part = m_upChunk->chunk()->at_position(m_upPiece.offset() + m_up->position());
+  Chunk::data_type data;
+  ChunkIterator itr(m_upChunk.chunk(), m_upPiece.offset(), m_upPiece.offset() + std::min(quota, m_upPiece.length()));
 
   do {
-    memory = m_upChunk->chunk()->at_memory(m_upPiece.offset() + m_up->position(), part++);
-    count = write_stream_throws(memory.first, std::min(left, memory.second));
+    data = itr.data();
+    data.second = write_stream_throws(data.first, data.second);
 
-    m_up->adjust_position(count);
-    left -= count;
+    bytesTransfered += data.second;
 
-  } while (count == memory.second && left != 0);
+  } while (itr.used(data.second));
 
-  uint32_t bytes = quota - left;
+  m_download->upload_throttle()->node_used(m_peerChunks.upload_throttle(), bytesTransfered);
+  m_download->info()->up_rate()->insert(bytesTransfered);
 
-  m_download->upload_throttle()->node_used(m_peerChunks.upload_throttle(), bytes);
-  m_download->info()->up_rate()->insert(bytes);
+  // Just modifying the piece to cover the remaining data ends up
+  // being much cleaner and we avoid an unnessesary position variable.
+  m_upPiece.set_offset(m_upPiece.offset() + bytesTransfered);
+  m_upPiece.set_length(m_upPiece.length() - bytesTransfered);
 
-  return m_up->position() == m_upPiece.length();
+  return m_upPiece.length() == 0;
 }
 
 void
 PeerConnectionBase::down_chunk_release() {
-  if (m_downChunk.is_valid()) {
-    m_download->chunk_list()->release(m_downChunk);
-    m_downChunk.clear();
-  }
+  if (m_downChunk.is_valid())
+    m_download->chunk_list()->release(&m_downChunk);
 }
 
 void
 PeerConnectionBase::up_chunk_release() {
-  if (m_upChunk.is_valid()) {
-    m_download->chunk_list()->release(m_upChunk);
-    m_upChunk.clear();
-  }
+  if (m_upChunk.is_valid())
+    m_download->chunk_list()->release(&m_upChunk);
 }
 
 void
@@ -361,9 +480,9 @@ PeerConnectionBase::read_cancel_piece(const Piece& p) {
 void
 PeerConnectionBase::read_buffer_move_unused() {
   uint32_t remaining = m_down->buffer()->remaining();
-	
+  
   std::memmove(m_down->buffer()->begin(), m_down->buffer()->position(), remaining);
-	
+  
   m_down->buffer()->reset_position();
   m_down->buffer()->set_end(remaining);
 }
@@ -390,41 +509,6 @@ PeerConnectionBase::write_prepare_piece() {
   m_up->write_piece(m_upPiece);
 }
 
-bool
-PeerConnectionBase::read_bitfield_body() {
-  // We're guaranteed that we still got bytes remaining to be
-  // read of the bitfield.
-  m_down->adjust_position(read_stream_throws(m_peerChunks.bitfield()->begin() + m_down->position(),
-					     m_peerChunks.bitfield()->size_bytes() - m_down->position()));
-	
-  return m_down->position() == m_peerChunks.bitfield()->size_bytes();
-}
-
-// 'msgLength' is the length of the message, not how much we got in
-// the buffer.
-bool
-PeerConnectionBase::read_bitfield_from_buffer(uint32_t msgLength) {
-  if (msgLength != m_peerChunks.bitfield()->size_bytes())
-    throw network_error("Received invalid bitfield size.");
-
-  uint32_t copyLength = std::min((uint32_t)m_down->buffer()->remaining(), msgLength);
-
-  std::memcpy(m_peerChunks.bitfield()->begin(), m_down->buffer()->position(), copyLength);
-
-  m_down->buffer()->move_position(copyLength);
-  m_down->set_position(copyLength);
-
-  return copyLength == msgLength;
-}
-
-bool
-PeerConnectionBase::write_bitfield_body() {
-  m_up->adjust_position(write_stream_throws(m_download->content()->bitfield()->begin() + m_up->position(),
-					    m_download->content()->bitfield()->size_bytes() - m_up->position()));
-
-  return m_up->position() == m_peerChunks.bitfield()->size_bytes();
-}
-
 // High stall count peers should request if we're *not* in endgame, or
 // if we're in endgame and the download is too slow. Prefere not to request
 // from high stall counts when we are doing decent speeds.
@@ -450,9 +534,20 @@ PeerConnectionBase::try_request_pieces() {
     m_downStall = 0;
 
   uint32_t pipeSize = download_queue()->calculate_pipe_size(m_peerChunks.download_throttle()->rate()->rate());
+
+  // Don't start requesting if we can't do it in large enough chunks.
+  if (download_queue()->size() >= (pipeSize + 10) / 2)
+    return false;
+
   bool success = false;
 
   while (download_queue()->size() < pipeSize && m_up->can_write_request()) {
+
+    // Delegator should return a vector of pieces, and it should be
+    // passed the number of pieces it should delegate. Try to ensure
+    // it receives large enough request to fill a whole chunk if the
+    // peer is fast enough.
+
     const Piece* p = download_queue()->delegate();
 
     if (p == NULL)
