@@ -73,7 +73,7 @@ public:
   PeerConnectionBase();
   virtual ~PeerConnectionBase();
   
-  void                initialize(DownloadMain* download, const PeerInfo& p, SocketFd fd);
+  void                initialize(DownloadMain* download, PeerInfo* p, SocketFd fd, Bitfield* bitfield);
 
   bool                is_up_choked()                { return m_up->choked(); }
   bool                is_up_interested()            { return m_up->interested(); }
@@ -84,7 +84,7 @@ public:
 
   bool                is_seeder() const             { return m_peerChunks.is_seeder(); }
 
-  const PeerInfo*     peer_info() const             { return &m_peer; }
+  const PeerInfo*     peer_info() const             { return m_peerInfo; }
   PeerChunks*         peer_chunks()                 { return &m_peerChunks; }
 
   RequestList*        download_queue()              { return &m_downloadQueue; }
@@ -108,13 +108,14 @@ public:
 
   virtual void        event_error();
 
+  void                push_unread(const void* data, uint32_t size);
+
 protected:
   typedef Chunk::iterator ChunkPart;
 
   inline bool         read_remaining();
   inline bool         write_remaining();
 
-  void                load_down_chunk(const Piece& p);
   void                load_up_chunk();
 
   void                receive_throttle_down_activate();
@@ -127,13 +128,17 @@ protected:
 
   void                write_prepare_piece();
 
-  bool                read_bitfield_body();
-  bool                read_bitfield_from_buffer(uint32_t msgLength);
-
-  bool                write_bitfield_body();
+  bool                down_chunk_start(const Piece& p);
+  void                down_chunk_finished();
 
   bool                down_chunk();
   bool                down_chunk_from_buffer();
+  bool                down_chunk_skip();
+  bool                down_chunk_skip_from_buffer();
+
+  uint32_t            down_chunk_process(const void* buffer, uint32_t length);
+  uint32_t            down_chunk_skip_process(const void* buffer, uint32_t length);
+
   bool                up_chunk();
 
   void                down_chunk_release();
@@ -154,12 +159,11 @@ protected:
   ProtocolRead*       m_down;
   ProtocolWrite*      m_up;
 
-  PeerInfo            m_peer;
+  PeerInfo*           m_peerInfo;
   PeerChunks          m_peerChunks;
 
-  Piece               m_downPiece;
+  RequestList         m_downloadQueue;
   ChunkHandle         m_downChunk;
-
   uint32_t            m_downStall;
 
   Piece               m_upPiece;
@@ -168,24 +172,26 @@ protected:
   bool                m_sendChoked;
   bool                m_sendInterested;
 
-  RequestList         m_downloadQueue;
-
   rak::timer          m_timeLastChoked;
   rak::timer          m_timeLastRead;
 };
 
+inline void
+PeerConnectionBase::push_unread(const void* data, uint32_t size) {
+  std::memcpy(m_down->buffer()->end(), data, size);
+  m_down->buffer()->move_end(size);
+}
+
 inline bool
 PeerConnectionBase::read_remaining() {
-  m_down->buffer()->move_position(read_stream_throws(m_down->buffer()->position(),
-						     m_down->buffer()->remaining()));
+  m_down->buffer()->move_position(read_stream_throws(m_down->buffer()->position(), m_down->buffer()->remaining()));
 
   return !m_down->buffer()->remaining();
 }
 
 inline bool
 PeerConnectionBase::write_remaining() {
-  m_up->buffer()->move_position(write_stream_throws(m_up->buffer()->position(),
-						m_up->buffer()->remaining()));
+  m_up->buffer()->move_position(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining()));
 
   return !m_up->buffer()->remaining();
 }
