@@ -58,19 +58,23 @@ PeerConnectionLeech::~PeerConnectionLeech() {
 
 void
 PeerConnectionLeech::initialize_custom() {
-  if (m_download->content()->chunks_completed() != 0) {
-    m_up->write_bitfield(m_download->content()->bitfield()->size_bytes());
+//   if (m_download->content()->chunks_completed() != 0) {
+//     m_up->write_bitfield(m_download->content()->bitfield()->size_bytes());
 
-    m_up->buffer()->prepare_end();
-    m_up->set_position(0);
-    m_up->set_state(ProtocolWrite::WRITE_BITFIELD_HEADER);
-  }
+//     m_up->buffer()->prepare_end();
+//     m_up->set_position(0);
+//     m_up->set_state(ProtocolWrite::WRITE_BITFIELD_HEADER);
+//   }
 }
 
 void
 PeerConnectionLeech::update_interested() {
 // FIXME:   if (m_download->delegator()->get_select().interested(m_peerChunks.bitfield()->bitfield())) {
 
+  // Consider just setting to interested without checking the
+  // bitfield. The status might change by the time we get unchoked
+  // anyway.
+
   if (true) {
     m_sendInterested = !m_up->interested();
     m_up->set_interested(true);
@@ -115,7 +119,7 @@ PeerConnectionLeech::receive_keepalive() {
   // should stay at zero or one when downloading at an acceptable
   // speed. Thus only when m_downStall >= 2 is the download actually
   // stalling.
-  if (!download_queue()->empty() && m_downStall++ > 0)
+  if (!download_queue()->empty() && m_downStall++ != 0)
     download_queue()->stall();
 
   return true;
@@ -199,21 +203,6 @@ PeerConnectionLeech::read_message() {
     read_have_chunk(buf->read_32());
     return true;
 
-  case ProtocolBase::BITFIELD:
-    // Bad peer, sending their bitfield after other messages have been
-    // sent.
-    if (m_peerChunks.using_counter() || !m_peerChunks.bitfield()->is_all_unset())
-      throw close_connection();
-
-    if (read_bitfield_from_buffer(length - 1)) {
-      finish_bitfield();
-      return true;
-
-    } else {
-      m_down->set_state(ProtocolRead::READ_BITFIELD);
-      return false;
-    }
-
   case ProtocolBase::REQUEST:
     if (!m_down->can_read_request_body())
       break;
@@ -232,42 +221,31 @@ PeerConnectionLeech::read_message() {
     if (!m_down->can_read_piece_body())
       break;
 
-    m_down->set_position(0);
-    m_downPiece = m_down->read_piece(length - 9);
+    if (!down_chunk_start(m_down->read_piece(length - 9))) {
 
-    if (!receive_piece_header()) {
-
-      // We're skipping this piece.
-      m_down->set_position(std::min<uint32_t>(buf->remaining(), m_downPiece.length()));
-      buf->move_position(m_down->position());
-
-      if (m_down->position() != m_downPiece.length()) {
-	m_down->set_state(ProtocolRead::READ_SKIP_PIECE);
-	return false;
+      // We don't want this chunk.
+      if (down_chunk_skip_from_buffer()) {
+        m_tryRequest = true;
+        down_chunk_finished();
+        return true;
 
       } else {
-	return true;
+        m_down->set_state(ProtocolRead::READ_SKIP_PIECE);
+        return false;
       }
       
-    } else if (down_chunk_from_buffer()) {
-      // Done with chunk.
-      m_downChunk->set_time_modified(cachedTime);
-      download_queue()->finished();
-    
-      if (m_downStall > 0)
-	m_downStall--;
-    
-      // TODO: clear m_down.data?
-      // TODO: remove throttle if choked? Rarely happens though.
-      m_tryRequest = true;
-      write_insert_poll_safe();
-
-      return true;
-
     } else {
-      m_down->set_state(ProtocolRead::READ_PIECE);
-      m_download->download_throttle()->insert(m_peerChunks.download_throttle());
-      return false;
+
+      if (down_chunk_from_buffer()) {
+        m_tryRequest = true;
+        down_chunk_finished();
+        return true;
+
+      } else {
+        m_down->set_state(ProtocolRead::READ_PIECE);
+        m_download->download_throttle()->insert(m_peerChunks.download_throttle());
+        return false;
+      }
     }
 
   case ProtocolBase::CANCEL:
@@ -310,69 +288,55 @@ PeerConnectionLeech::event_read() {
 
       switch (m_down->get_state()) {
       case ProtocolRead::IDLE:
-	m_down->buffer()->move_end(read_stream_throws(m_down->buffer()->end(), read_size - m_down->buffer()->size_end()));
-	
-	while (read_message());
-	
-	if (m_down->buffer()->size_end() == read_size) {
-	  read_buffer_move_unused();
-	  break;
-	} else {
-	  read_buffer_move_unused();
-	  return;
-	}
+        if (m_down->buffer()->size_end() == read_size)
+          throw internal_error("PeerConnectionLeech::event_read() m_down->buffer()->size_end() == read_size.");
+
+        m_down->buffer()->move_end(read_stream_throws(m_down->buffer()->end(), read_size - m_down->buffer()->size_end()));
+        
+        while (read_message());
+        
+        if (m_down->buffer()->size_end() == read_size) {
+          read_buffer_move_unused();
+          break;
+        } else {
+          read_buffer_move_unused();
+          return;
+        }
 
       case ProtocolRead::READ_PIECE:
-	if (!download_queue()->is_downloading())
-	  throw internal_error("ProtocolRead::READ_PIECE state but RequestList is not downloading");
+        if (!download_queue()->is_downloading())
+          throw internal_error("ProtocolRead::READ_PIECE state but RequestList is not downloading");
 
-	if (!download_queue()->is_wanted()) {
-	  m_down->set_state(ProtocolRead::READ_SKIP_PIECE);
-	  download_queue()->skip();
+        if (!m_downloadQueue.transfer()->is_valid() || !m_downloadQueue.transfer()->is_leader()) {
+          m_down->set_state(ProtocolRead::READ_SKIP_PIECE);
+          break;
+        }
 
-	  break;
-	}
+        if (!down_chunk())
+          return;
 
-	if (!down_chunk())
-	  return;
-
-	m_downChunk->set_time_modified(cachedTime);
-	download_queue()->finished();
-	
-	if (m_downStall > 0)
-	  m_downStall--;
-	
-	// TODO: clear m_down.data?
-	// TODO: remove throttle if choked? Rarely happens though.
-	m_tryRequest = true;
-	m_down->set_state(ProtocolRead::IDLE);
-	write_insert_poll_safe();
-	
-	break;
+        m_tryRequest = true;
+        m_down->set_state(ProtocolRead::IDLE);
+        down_chunk_finished();
+        break;
 
       case ProtocolRead::READ_SKIP_PIECE:
-	m_down->adjust_position(ignore_stream_throws(m_downPiece.length() - m_down->position()));
+        if (download_queue()->transfer()->is_leader()) {
+          m_down->set_state(ProtocolRead::READ_PIECE);
+          m_download->download_throttle()->insert(m_peerChunks.download_throttle());
+          break;
+        }
 
-	if (m_down->position() != m_downPiece.length())
-	  return;
+        if (!down_chunk_skip())
+          return;
 
-	m_down->set_state(ProtocolRead::IDLE);
-	m_down->buffer()->reset();
-	
-	break;
-
-      case ProtocolRead::READ_BITFIELD:
-	if (!read_bitfield_body())
-	  return;
-
-	m_down->set_state(ProtocolRead::IDLE);
-	m_down->buffer()->reset();
-
-	finish_bitfield();
-	break;
+        m_tryRequest = true;
+        m_down->set_state(ProtocolRead::IDLE);
+        down_chunk_finished();
+        break;
 
       default:
-	throw internal_error("PeerConnectionLeech::event_read() wrong state.");
+        throw internal_error("PeerConnectionLeech::event_read() wrong state.");
       }
 
       // Figure out how to get rid of the shouldLoop boolean.
@@ -462,69 +426,51 @@ PeerConnectionLeech::event_write() {
       switch (m_up->get_state()) {
       case ProtocolWrite::IDLE:
 
-	// We might have buffered keepalive message or similar, but
-	// 'end' should remain at the start of the buffer.
-	if (m_up->buffer()->size_end() != 0)
-	  throw internal_error("PeerConnectionLeech::event_write() ProtocolWrite::IDLE in a wrong state.");
+        // We might have buffered keepalive message or similar, but
+        // 'end' should remain at the start of the buffer.
+        if (m_up->buffer()->size_end() != 0)
+          throw internal_error("PeerConnectionLeech::event_write() ProtocolWrite::IDLE in a wrong state.");
 
-	// Fill up buffer.
-	fill_write_buffer();
+        // Fill up buffer.
+        fill_write_buffer();
 
-	if (m_up->buffer()->size_position() == 0) {
-	  manager->poll()->remove_write(this);
-	  return;
-	}
+        if (m_up->buffer()->size_position() == 0) {
+          manager->poll()->remove_write(this);
+          return;
+        }
 
-	m_up->set_state(ProtocolWrite::MSG);
-	m_up->buffer()->prepare_end();
+        m_up->set_state(ProtocolWrite::MSG);
+        m_up->buffer()->prepare_end();
 
       case ProtocolWrite::MSG:
-	m_up->buffer()->move_position(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining()));
+        m_up->buffer()->move_position(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining()));
 
-	if (m_up->buffer()->remaining())
-	  return;
+        if (m_up->buffer()->remaining())
+          return;
 
-	m_up->buffer()->reset();
+        m_up->buffer()->reset();
 
-	if (m_up->last_command() != ProtocolBase::PIECE) {
-	  // Break or loop? Might do an ifelse based on size of the
-	  // write buffer. Also the write buffer is relatively large.
-	  m_up->set_state(ProtocolWrite::IDLE);
-	  break;
-	}
+        if (m_up->last_command() != ProtocolBase::PIECE) {
+          // Break or loop? Might do an ifelse based on size of the
+          // write buffer. Also the write buffer is relatively large.
+          m_up->set_state(ProtocolWrite::IDLE);
+          break;
+        }
 
-	// We're uploading a piece.
-	load_up_chunk();
-
-	m_up->set_state(ProtocolWrite::WRITE_PIECE);
-	m_up->set_position(0);
+        // We're uploading a piece.
+        load_up_chunk();
+        m_up->set_state(ProtocolWrite::WRITE_PIECE);
 
       case ProtocolWrite::WRITE_PIECE:
-	if (!up_chunk())
-	  return;
+        if (!up_chunk())
+          return;
 
-	m_up->set_state(ProtocolWrite::IDLE);
+        m_up->set_state(ProtocolWrite::IDLE);
 
-	break;
-
-      case ProtocolWrite::WRITE_BITFIELD_HEADER:
-	m_up->buffer()->move_position(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining()));
-
-	if (m_up->buffer()->remaining())
-	  return;
-
-	m_up->buffer()->reset();
-	m_up->set_state(ProtocolWrite::WRITE_BITFIELD);
-
-      case ProtocolWrite::WRITE_BITFIELD:
-	if (!write_bitfield_body())
-	  return;
-
-	m_up->set_state(ProtocolWrite::IDLE);
-	break;
+        break;
 
       default:
-	throw internal_error("PeerConnectionLeech::event_write() wrong state.");
+        throw internal_error("PeerConnectionLeech::event_write() wrong state.");
       }
 
     } while (true);
@@ -595,38 +541,4 @@ PeerConnectionLeech::read_have_chunk(uint32_t index) {
   }
 }
 
-void
-PeerConnectionLeech::finish_bitfield() {
-  m_peerChunks.bitfield()->update();
-
-  if (m_download->content()->is_done() && m_peerChunks.bitfield()->is_all_set())
-    throw close_connection();
-
-  m_download->chunk_statistics()->received_connect(&m_peerChunks);
-
-  // By default, set interested. When unchoked we'll check if there's
-  // any interesting pieces, then set uninterested.
-  if (!m_download->content()->is_done()) {
-    m_sendInterested = true;
-    m_up->set_interested(true);
-  }
-
-  write_insert_poll_safe();
-}
-
-bool
-PeerConnectionLeech::receive_piece_header() {
-  if (download_queue()->downloading(m_downPiece)) {
-
-    load_down_chunk(m_downPiece);
-    return true;
-
-  } else {
-    if (m_downPiece.length() == 0)
-      m_download->info()->signal_network_log().emit("Received piece with length zero");
-
-    return false;
-  }
-}
-
 }
