@@ -67,15 +67,11 @@ public:
     MSG,
     READ_PIECE,
     READ_SKIP_PIECE,
-    READ_BITFIELD,
-    WRITE_BITFIELD_HEADER,
-    WRITE_BITFIELD,
     WRITE_PIECE,
     INTERNAL_ERROR
   } State;
 
   ProtocolBase() :
-    m_position(0),
     m_choked(true),
     m_interested(false),
     m_state(IDLE),
@@ -95,12 +91,6 @@ public:
 
   Buffer*             buffer()                                { return &m_buffer; }
 
-  // Position should perhaps be in a different place, like a dedicated
-  // chunk writing class.
-  uint32_t            position() const                        { return m_position; }
-  void                set_position(size_type p)               { m_position = p; }
-  void                adjust_position(size_type p)            { m_position += p; }
-
   State               get_state() const                       { return m_state; }
   void                set_state(State s)                      { m_state = s; }
 
@@ -142,8 +132,6 @@ public:
   bool                can_read_piece_body() const             { return m_buffer.remaining() >= sizeof_piece_body; }
 
 protected:
-  uint32_t            m_position;
-
   bool                m_choked;
   bool                m_interested;
 
