@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005, Jari Sundell
+// Copyright (C) 2005-2006, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -34,15 +34,19 @@
 //           Skomakerveien 33
 //           3185 Skoppum, NORWAY
 
-#ifndef LIBTORRENT_COMMON_H
-#define LIBTORRENT_COMMON_H
+#ifndef LIBTORRENT_PRIORITY_H
+#define LIBTORRENT_PRIORITY_H
 
-#include <string>
 #include <inttypes.h>
+#include <rak/ranges.h>
 
 namespace torrent {
 
-class Path;
+typedef enum {
+  PRIORITY_OFF = 0,
+  PRIORITY_NORMAL,
+  PRIORITY_HIGH
+} priority_t;
 
 }
 
