@@ -40,12 +40,12 @@
 #include <sigc++/bind.h>
 #include <sigc++/hide.h>
 
+#include "data/chunk_list.h"
 #include "data/hash_queue.h"
 #include "data/hash_torrent.h"
 #include "download/choke_manager.h"
 #include "download/chunk_selector.h"
 #include "download/chunk_statistics.h"
-#include "download/delegator_chunk.h"
 #include "download/download_wrapper.h"
 #include "protocol/peer_connection_base.h"
 #include "protocol/peer_factory.h"
@@ -53,6 +53,8 @@
 #include "tracker/tracker_manager.h"
 
 #include "exceptions.h"
+#include "block.h"
+#include "block_list.h"
 #include "download.h"
 #include "file_list.h"
 #include "object.h"
@@ -75,7 +77,32 @@ Download::close() {
 
 void
 Download::start() {
-  m_ptr->start();
+  if (!m_ptr->hash_checker()->is_checked())
+    throw client_error("Tried to start an unchecked download");
+
+  if (!m_ptr->main()->is_open())
+    throw client_error("Tried to start a closed download");
+
+  if (m_ptr->main()->is_active())
+    return;
+
+  m_ptr->main()->start();
+  m_ptr->main()->tracker_manager()->set_active(true);
+
+  // Either the client queued a completed request, or it is still
+  // sending the stopped request. Don't send started nor reset the
+  // baseline.
+  //
+  // Check for stopped request.
+  if (m_ptr->main()->tracker_manager()->is_busy())
+    return;
+
+  // Reset the uploaded/download baseline when we restart the download
+  // so that broken trackers get the right uploaded ratio.
+  m_ptr->info()->set_uploaded_baseline(m_ptr->info()->up_rate()->total());
+  m_ptr->info()->set_downloaded_baseline(m_ptr->info()->down_rate()->total());
+
+  m_ptr->main()->tracker_manager()->send_start();
 }
 
 void
@@ -83,30 +110,53 @@ Download::stop() {
   if (!m_ptr->main()->is_active())
     return;
 
-  m_ptr->stop();
+  m_ptr->main()->stop();
+  m_ptr->main()->tracker_manager()->set_active(false);
+  m_ptr->main()->tracker_manager()->send_stop();
 }
 
-void
-Download::hash_check(bool resume) {
-  if (m_ptr->hash_checker()->is_checked() ||
-      m_ptr->hash_checker()->is_checking())
-    throw client_error("Download::hash_check(...) called but already checking or complete.");
+bool
+Download::hash_check(bool tryQuick) {
+  if (m_ptr->hash_checker()->is_checking())
+    return m_ptr->hash_checker()->start(tryQuick);
 
-  if (resume)
-    m_ptr->hash_resume_load();
+  if (!m_ptr->main()->is_open() || m_ptr->main()->is_active())
+    throw client_error("Download::hash_check(...) called on a closed or active download.");
 
-  m_ptr->hash_checker()->start();
+  if (m_ptr->hash_checker()->is_checked())
+    throw client_error("Download::hash_check(...) called but already hash checked.");
+
+  // The bitfield still hasn't been allocated, so no resume data was
+  // given. 
+
+  if (m_ptr->main()->content()->bitfield()->empty()) {
+    m_ptr->main()->content()->bitfield()->allocate();
+    m_ptr->main()->content()->bitfield()->unset_all();
+
+    m_ptr->hash_checker()->ranges().insert(0, m_ptr->main()->content()->chunk_total());
+
+    // Make sure other book-keeping is cleared, like file progress.
+
+  } else {
+    m_ptr->main()->content()->update_done();
+  }
+
+  return m_ptr->hash_checker()->start(tryQuick);
 }
 
+// Propably not correct, need to clear content, etc.
 void
-Download::hash_resume_save() {
-  m_ptr->hash_resume_save();
-}
+Download::hash_stop() {
+  if (!m_ptr->hash_checker()->is_checking())
+    return;
 
-void
-Download::hash_resume_clear() {
-  if (m_ptr->bencode()->has_key("libtorrent resume"))
-    m_ptr->bencode()->get_key("libtorrent resume").erase_key("bitfield");
+  // Stop the hashing first as we need to make sure all chunks are
+  // released when DownloadMain::close() is called.
+  m_ptr->hash_checker()->clear();
+
+  // Clear after m_hash to ensure that the empty hash done signal does
+  // not get passed to HashTorrent.
+  m_ptr->hash_checker()->get_queue()->remove(m_ptr);
 }
 
 bool
@@ -121,7 +171,7 @@ Download::is_active() const {
 
 bool
 Download::is_hash_checked() const {
-  return m_ptr->main()->is_open() && m_ptr->hash_checker()->is_checked();
+  return m_ptr->hash_checker()->is_checked();
 }
 
 bool
@@ -156,7 +206,7 @@ Download::local_id() const {
 uint32_t
 Download::creation_date() const {
   if (m_ptr->bencode()->has_key_value("creation date"))
-    return m_ptr->bencode()->get_key("creation date").as_value();
+    return m_ptr->bencode()->get_key_value("creation date");
   else
     return 0;
 }
@@ -181,6 +231,11 @@ Download::tracker_list() const {
   return TrackerList(m_ptr->main()->tracker_manager());
 }
 
+const TransferList*
+Download::transfer_list() const {
+  return m_ptr->main()->delegator()->transfer_list();
+}
+
 Rate*
 Download::down_rate() {
   return m_ptr->info()->down_rate();
@@ -207,10 +262,10 @@ Download::bytes_done() const {
  
   Delegator* d = m_ptr->main()->delegator();
 
-  for (Delegator::Chunks::iterator itr1 = d->get_chunks().begin(), last1 = d->get_chunks().end(); itr1 != last1; ++itr1)
-    for (DelegatorChunk::iterator itr2 = (*itr1)->begin(), last2 = (*itr1)->end(); itr2 != last2; ++itr2)
+  for (TransferList::const_iterator itr1 = d->transfer_list()->begin(), last1 = d->transfer_list()->end(); itr1 != last1; ++itr1)
+    for (BlockList::const_iterator itr2 = (*itr1)->begin(), last2 = (*itr1)->end(); itr2 != last2; ++itr2)
       if (itr2->is_finished())
-	a += itr2->get_piece().length();
+        a += itr2->piece().length();
   
   return a + m_ptr->main()->content()->bytes_completed();
 }
@@ -245,11 +300,88 @@ Download::chunks_seen() const {
   return !m_ptr->main()->chunk_statistics()->empty() ? &*m_ptr->main()->chunk_statistics()->begin() : NULL;
 }
 
+void
+Download::set_chunks_done(uint32_t chunks) {
+  if (m_ptr->main()->is_open())
+    throw input_error("Download::set_chunks_done(...) Download is open.");
+
+  m_ptr->main()->content()->bitfield()->set_size_set(chunks);
+}
+
+void
+Download::set_bitfield(uint8_t* first, uint8_t* last) {
+  if (m_ptr->hash_checker()->is_checked() || m_ptr->hash_checker()->is_checking())
+    throw input_error("Download::set_bitfield(...) Download in invalid state.");
+
+  if (std::distance(first, last) != (ptrdiff_t)m_ptr->main()->content()->bitfield()->size_bytes())
+    throw input_error("Download::set_bitfield(...) Invalid length.");
+
+  m_ptr->main()->content()->bitfield()->allocate();
+  std::memcpy(m_ptr->main()->content()->bitfield()->begin(), first, m_ptr->main()->content()->bitfield()->size_bytes());
+  m_ptr->main()->content()->bitfield()->update();
+  
+  m_ptr->hash_checker()->ranges().clear();
+}
+
+void
+Download::clear_range(uint32_t first, uint32_t last) {
+  if (m_ptr->hash_checker()->is_checked() || m_ptr->hash_checker()->is_checking() || m_ptr->main()->content()->bitfield()->empty())
+    throw input_error("Download::clear_range(...) Download in invalid state.");
+
+  m_ptr->hash_checker()->ranges().insert(first, last);
+  m_ptr->main()->content()->bitfield()->unset_range(first, last);
+}
+ 
 const Bitfield*
 Download::bitfield() const {
   return m_ptr->main()->content()->bitfield();
 }
 
+void
+Download::sync_chunks() {
+  m_ptr->main()->chunk_list()->sync_all(MemoryChunk::sync_sync);
+}
+
+void
+Download::insert_addresses(const std::string& addresses) {
+  m_ptr->insert_available_list(addresses);
+}
+
+void
+Download::extract_addresses(std::string& addresses) {
+  m_ptr->extract_available_list(addresses);
+}
+
+uint32_t
+Download::timeout_sync() const {
+  return m_ptr->main()->chunk_list()->timeout_sync();
+}
+
+void
+Download::set_timeout_sync(uint32_t seconds) {
+  m_ptr->main()->chunk_list()->set_timeout_sync(seconds);
+}
+
+uint32_t
+Download::timeout_safe_sync() const {
+  return m_ptr->main()->chunk_list()->timeout_safe_sync();
+}
+
+void
+Download::set_timeout_safe_sync(uint32_t seconds) {
+  m_ptr->main()->chunk_list()->set_timeout_safe_sync(seconds);
+}
+
+uint32_t
+Download::max_chunks_queued() const {
+  return m_ptr->main()->chunk_list()->max_queue_size();
+}
+
+void
+Download::set_max_chunks_queued(uint32_t chunks) {
+  m_ptr->main()->chunk_list()->set_max_queue_size(chunks);
+}
+
 uint32_t
 Download::peers_min() const {
   return m_ptr->main()->connection_list()->get_min_size();
