@@ -57,6 +57,7 @@ class Rate;
 class Object;
 class Peer;
 class TrackerList;
+class TransferList;
 
 // Download is safe to copy and destory as it is just a pointer to an
 // internal class.
@@ -71,12 +72,14 @@ public:
   void                open();
   void                close();
 
-  // Torrent must be open for calls to hash_check(bool) and
-  // hash_resume_save(). hash_resume_clear() removes resume data from
-  // the bencode'ed torrent.
-  void                hash_check(bool resume = true);
-  void                hash_resume_save();
-  void                hash_resume_clear();
+  // When 'tryQuick' is true, it will only check if the chunks can be
+  // mmaped and returns if one is encountered. If it doesn't find any
+  // mappable chunks it will return true to indicate that it is
+  // finished and a hash done signal has been queued.
+  //
+  // Chunk ranges that have valid resume data won't be checked.
+  bool                hash_check(bool tryQuick);
+  void                hash_stop();
 
   // Start/stop the download. The torrent must be open.
   void                start();
@@ -105,6 +108,8 @@ public:
   FileList            file_list() const;
   TrackerList         tracker_list() const;
 
+  const TransferList* transfer_list() const;
+
   Rate*               down_rate();
   const Rate*         down_rate() const;
 
@@ -123,8 +128,40 @@ public:
 
   const uint8_t*      chunks_seen() const;
 
+  // Set the number of finished chunks for closed torrents.
+  void                set_chunks_done(uint32_t chunks);
+
+  // Use the below to set the resume data and what chunk ranges need
+  // to be hash checked. If they arn't called then by default it will
+  // use an cleared bitfield and check the whole range.
+  //
+  // These must be called when is_open, !is_checked and !is_checking.
+  void                set_bitfield(uint8_t* first, uint8_t* last);
+  void                clear_range(uint32_t first, uint32_t last);
+
   const Bitfield*     bitfield() const;
 
+  // Temporary hack for syncing chunks to disk before hash resume is
+  // saved.
+  void                sync_chunks();
+
+  // Temporary hack until i can move the available list stuf somewhere
+  // and make a nice interface for it.
+
+  void                insert_addresses(const std::string& addresses);
+  void                extract_addresses(std::string& addresses);
+
+  // This should also be moved out into a seperate class, possibly
+  // global settings.
+  uint32_t            timeout_sync() const;
+  void                set_timeout_sync(uint32_t seconds);
+
+  uint32_t            timeout_safe_sync() const;
+  void                set_timeout_safe_sync(uint32_t seconds);
+
+  uint32_t            max_chunks_queued() const;
+  void                set_max_chunks_queued(uint32_t chunks);
+
   uint32_t            peers_min() const;
   uint32_t            peers_max() const;
   uint32_t            peers_connected() const;
