@@ -40,6 +40,8 @@
 #include "protocol/peer_connection_base.h"
 
 #include "exceptions.h"
+#include "block.h"
+#include "block_transfer.h"
 #include "peer.h"
 #include "rate.h"
 
@@ -115,12 +117,16 @@ Peer::bitfield() const {
   return m_ptr->peer_chunks()->bitfield();
 }
 
-const Piece*
-Peer::incoming_queue(uint32_t pos) const {
-  if (pos >= m_ptr->download_queue()->size())
-    throw client_error("Peer::incoming_queue(...) out of range.");
+const BlockTransfer*
+Peer::transfer() const {
+  if (m_ptr->download_queue()->transfer() != NULL)
+    return m_ptr->download_queue()->transfer();
 
-  return m_ptr->download_queue()->queued_piece(pos);
+  else if (!m_ptr->download_queue()->empty())
+    return m_ptr->download_queue()->queued_transfer(0);
+
+  else
+    return NULL;
 }
 
 uint32_t
@@ -138,4 +144,9 @@ Peer::chunks_done() const {
   return m_ptr->peer_chunks()->bitfield()->size_set();
 }  
 
+uint32_t
+Peer::failed_counter() const {
+  return m_ptr->peer_info()->failed_counter();
+}  
+
 }
