@@ -36,19 +36,20 @@
 
 #include "config.h"
 
-#include <iostream>
+#include <rak/address_info.h>
 #include <rak/functional.h>
 #include <rak/string_manip.h>
 
 #include "exceptions.h"
 #include "torrent.h"
-#include "bencode.h"
+#include "object.h"
+#include "object_stream.h"
+#include "connection_manager.h"
+#include "poll.h"
 
 #include "manager.h"
 #include "resource_manager.h"
 
-#include "net/listen.h"
-#include "net/manager.h"
 #include "net/throttle_list.h"
 #include "net/throttle_manager.h"
 #include "protocol/handshake_manager.h"
@@ -95,18 +96,17 @@ initialize(Poll* poll) {
   if (manager != NULL)
     throw client_error("torrent::initialize(...) called but the library has already been initialized");
 
-  cachedTime = rak::timer::current();
-
-  manager = new Manager;
-
-  pollCustom = poll;
-
   if (poll->open_max() < 64)
     throw client_error("Could not initialize libtorrent, Poll::open_max() < 64.");
 
+  cachedTime = rak::timer::current();
+
+  manager = new Manager;
+  manager->set_poll(poll);
+
   uint32_t maxFiles = calculate_max_open_files(poll->open_max());
 
-  socketManager.set_max_size(poll->open_max() - maxFiles - calculate_reserved(poll->open_max()));
+  manager->connection_manager()->set_max_size(poll->open_max() - maxFiles - calculate_reserved(poll->open_max()));
   manager->file_manager()->set_max_size(maxFiles);
 }
 
@@ -119,35 +119,6 @@ cleanup() {
 
   delete manager;
   manager = NULL;
-
-  pollCustom = NULL;
-}
-
-bool
-listen_open(uint16_t begin, uint16_t end) {
-  if (manager == NULL)
-    throw client_error("listen_open called but the library has not been initialized");
-
-  if (!manager->listen()->open(begin, end, *manager->bind_address()))
-    return false;
-
-  manager->local_address()->set_port(manager->listen()->port());
-  manager->handshake_manager()->set_bind_address(*manager->bind_address());
-
-  for (DownloadManager::const_iterator itr = manager->download_manager()->begin(), last = manager->download_manager()->end(); itr != last; ++itr)
-    (*itr)->local_address().set_port(manager->listen()->port());
-
-  return true;
-}
-
-void
-listen_close() {
-  manager->listen()->close();
-}
-
-uint16_t
-listen_port() {
-  return manager->listen()->port();
 }
 
 void
@@ -171,44 +142,9 @@ is_inactive() {
     == manager->download_manager()->end();
 }
 
-std::string
-local_address() {
-  if (!manager->local_address()->is_address_any())
-    return manager->local_address()->get_address();
-  else
-    return std::string();
-}
-
-void
-set_local_address(const std::string& addr) {
-  if (!manager->local_address()->set_hostname(addr))
-    throw input_error("Tried to set an invalid/non-existent local address.");
-
-  for (DownloadManager::const_iterator itr = manager->download_manager()->begin(), last = manager->download_manager()->end(); itr != last; ++itr)
-    (*itr)->local_address() = *manager->local_address();
-}
-
-std::string
-bind_address() {
-  if (!manager->bind_address()->is_address_any())
-    return manager->bind_address()->get_address();
-  else
-    return std::string();
-}
-
-void
-set_bind_address(const std::string& addr) {
-  if (manager->listen()->is_open())
-    throw input_error("Tried to set the bind address while the listening socket is open.");
-
-  if (addr.empty())
-    manager->bind_address()->set_address_any();
-
-  else if (!manager->bind_address()->set_hostname(addr))
-    throw input_error("Tried to set an invalid/non-existent bind address.");
-
-  for (DownloadManager::const_iterator itr = manager->download_manager()->begin(), last = manager->download_manager()->end(); itr != last; ++itr)
-    (*itr)->bind_address() = *manager->bind_address();
+ConnectionManager*
+connection_manager() {
+  return manager->connection_manager();
 }
 
 uint32_t
@@ -344,12 +280,12 @@ set_max_open_files(uint32_t size) {
 
 uint32_t
 open_sockets() {
-  return socketManager.size();
+  return manager->connection_manager()->size();
 }
 
 uint32_t
 max_open_sockets() {
-  return socketManager.max_size();
+  return manager->connection_manager()->max_size();
 }
 
 void
@@ -357,7 +293,7 @@ set_max_open_sockets(uint32_t size) {
   if (size < 4 || size > (1 << 16))
     throw input_error("Max open sockets must be between 4 and 2^16.");
 
-  socketManager.set_max_size(size);
+  manager->connection_manager()->set_max_size(size);
 }
 
 EncodingList*
@@ -366,35 +302,28 @@ encoding_list() {
 }
 
 Download
-download_add(std::istream* s) {
-  if (!s->good())
-    throw input_error("Could not create download, the input stream is not valid.");
+download_add(Object* object) {
+  std::auto_ptr<DownloadWrapper> download(new DownloadWrapper);
 
-  std::auto_ptr<DownloadWrapper> d(new DownloadWrapper);
-
-  *s >> d->bencode();
-
-  if (s->fail())
-    throw input_error("Could not create download, the input is not a valid torrent.");
-  
   DownloadConstructor ctor;
-  ctor.set_download(d.get());
+  ctor.set_download(download.get());
   ctor.set_encoding_list(manager->encoding_list());
 
-  ctor.initialize(d->bencode());
+  ctor.initialize(*object);
 
-  d->initialize(d->bencode().get_key("info").compute_sha1(),
-		PEER_NAME + rak::generate_random<std::string>(20 - std::string(PEER_NAME).size()));
+  download->initialize(object_sha1(&object->get_key("info")),
+		       PEER_NAME + rak::generate_random<std::string>(20 - std::string(PEER_NAME).size()));
 
   // Default PeerConnection factory functions.
-  d->main()->connection_list()->slot_new_connection(&createPeerConnectionDefault);
+  download->main()->connection_list()->slot_new_connection(&createPeerConnectionDefault);
 
   // Consider move as much as possible into this function
   // call. Anything that won't cause possible torrent creation errors
   // go in there.
-  manager->initialize_download(d.get());
+  manager->initialize_download(download.get());
 
-  return Download(d.release());
+  download->set_bencode(object);
+  return Download(download.release());
 }
 
 void
