@@ -36,17 +36,19 @@
 
 #include "config.h"
 
-#include <stdlib.h>
+#include <rak/address_info.h>
 #include <rak/error_number.h>
 
-#include "net/manager.h"
 #include "torrent/exceptions.h"
+#include "torrent/connection_manager.h"
+#include "torrent/poll.h"
 
 #include "tracker_udp.h"
+#include "manager.h"
 
 namespace torrent {
 
-TrackerUdp::TrackerUdp(TrackerInfo* info, const std::string& url) :
+TrackerUdp::TrackerUdp(DownloadInfo* info, const std::string& url) :
   TrackerBase(info, url),
   m_readBuffer(NULL),
   m_writeBuffer(NULL) {
@@ -64,10 +66,7 @@ TrackerUdp::is_busy() const {
 }
 
 void
-TrackerUdp::send_state(TrackerInfo::State state,
-		       uint64_t down,
-		       uint64_t up,
-		       uint64_t left) {
+TrackerUdp::send_state(DownloadInfo::State state, uint64_t down, uint64_t up, uint64_t left) {
   close();
 
   if (!parse_url())
@@ -75,7 +74,7 @@ TrackerUdp::send_state(TrackerInfo::State state,
 
   if (!get_fd().open_datagram() ||
       !get_fd().set_nonblock() ||
-      !get_fd().bind(m_info->bind_address()))
+      !get_fd().bind(*rak::socket_address::cast_from(manager->connection_manager()->bind_address())))
     return receive_failed("Could not open UDP socket.");
 
   m_readBuffer = new ReadBuffer;
@@ -88,10 +87,10 @@ TrackerUdp::send_state(TrackerInfo::State state,
 
   prepare_connect_input();
 
-  pollCustom->open(this);
-  pollCustom->insert_read(this);
-  pollCustom->insert_write(this);
-  pollCustom->insert_error(this);
+  manager->poll()->open(this);
+  manager->poll()->insert_read(this);
+  manager->poll()->insert_write(this);
+  manager->poll()->insert_error(this);
 
   m_tries = m_info->udp_tries();
   priority_queue_insert(&taskScheduler, &m_taskTimeout, (cachedTime + rak::timer::from_seconds(m_info->udp_timeout())).round_seconds());
@@ -110,17 +109,17 @@ TrackerUdp::close() {
 
   priority_queue_erase(&taskScheduler, &m_taskTimeout);
 
-  pollCustom->remove_read(this);
-  pollCustom->remove_write(this);
-  pollCustom->remove_error(this);
-  pollCustom->close(this);
+  manager->poll()->remove_read(this);
+  manager->poll()->remove_write(this);
+  manager->poll()->remove_error(this);
+  manager->poll()->close(this);
 
   get_fd().close();
   get_fd().clear();
 }
 
 TrackerUdp::Type
-TrackerUdp::get_type() const {
+TrackerUdp::type() const {
   return TRACKER_UDP;
 }
 
@@ -140,22 +139,28 @@ TrackerUdp::receive_timeout() {
   } else {
     priority_queue_insert(&taskScheduler, &m_taskTimeout, (cachedTime + rak::timer::from_seconds(m_info->udp_timeout())).round_seconds());
 
-    pollCustom->insert_write(this);
+    manager->poll()->insert_write(this);
   }
 }
 
 void
 TrackerUdp::event_read() {
-  SocketAddress sa;
+  rak::socket_address sa;
 
   int s = read_datagram(m_readBuffer->begin(), m_readBuffer->reserved(), &sa);
 
-  if (s < 4)
+  if (s < 0)
     return;
 
   m_readBuffer->reset_position();
   m_readBuffer->set_end(s);
 
+  if (!m_info->signal_tracker_dump().empty())
+    m_info->signal_tracker_dump().emit(m_url, (const char*)m_readBuffer->begin(), s);
+
+  if (s < 4)
+    return;
+
   // Make sure sa is from the source we expected?
 
   // Do something with the content here.
@@ -170,7 +175,7 @@ TrackerUdp::event_read() {
     priority_queue_insert(&taskScheduler, &m_taskTimeout, (cachedTime + rak::timer::from_seconds(m_info->udp_timeout())).round_seconds());
 
     m_tries = m_info->udp_tries();
-    pollCustom->insert_write(this);
+    manager->poll()->insert_write(this);
     return;
 
   case 1:
@@ -201,7 +206,7 @@ TrackerUdp::event_write() {
   if (s != m_writeBuffer->size_end())
     ;
 
-  pollCustom->remove_write(this);
+  manager->poll()->remove_write(this);
 }
 
 void
@@ -211,17 +216,26 @@ TrackerUdp::event_error() {
 bool
 TrackerUdp::parse_url() {
   int port;
-  char hostname[256];
+  char hostname[1024];
       
-  if (std::sscanf(m_url.c_str(), "udp://%255[^:]:%i", hostname, &port) != 2 ||
+  if (std::sscanf(m_url.c_str(), "udp://%1023[^:]:%i", hostname, &port) != 2 ||
       hostname[0] == '\0' ||
       port <= 0 || port >= (1 << 16))
     return false;
 
-  m_connectAddress.set_hostname(hostname);
+  int err;
+  rak::address_info* ai;
+
+  if ((err = rak::address_info::get_address_info(hostname, PF_INET, SOCK_STREAM, &ai)) != 0)
+    return false;
+  
+  if (ai->address()->family() != rak::socket_address::af_inet)
+    return false;
+
+  m_connectAddress.copy(*ai->address(), ai->length());
   m_connectAddress.set_port(port);
 
-  return !m_connectAddress.is_port_any() && !m_connectAddress.is_address_any();
+  return m_connectAddress.is_valid();
 }
 
 void
@@ -242,18 +256,24 @@ TrackerUdp::prepare_announce_input() {
   m_writeBuffer->write_32(m_action = 1);
   m_writeBuffer->write_32(m_transactionId = random());
 
-  m_writeBuffer->write_range(m_info->get_hash().begin(), m_info->get_hash().end());
-  m_writeBuffer->write_range(m_info->get_local_id().begin(), m_info->get_local_id().end());
+  m_writeBuffer->write_range(m_info->hash().begin(), m_info->hash().end());
+  m_writeBuffer->write_range(m_info->local_id().begin(), m_info->local_id().end());
 
   m_writeBuffer->write_64(m_sendDown);
   m_writeBuffer->write_64(m_sendLeft);
   m_writeBuffer->write_64(m_sendUp);
   m_writeBuffer->write_32(m_sendState);
 
-  m_writeBuffer->write_32(m_info->local_address().get_addr_in_addr());
-  m_writeBuffer->write_32(m_info->get_key());
-  m_writeBuffer->write_32(m_info->get_numwant());
-  m_writeBuffer->write_16(m_info->local_address().get_port());
+  const rak::socket_address* localAddress = rak::socket_address::cast_from(manager->connection_manager()->local_address());
+
+  // This code assumes we're have a inet address.
+  if (localAddress->family() != rak::socket_address::af_inet)
+    throw internal_error("TrackerUdp::prepare_announce_input() m_info->local_address() not of family AF_INET.");
+
+  m_writeBuffer->write_32_n(localAddress->sa_inet()->address_n());
+  m_writeBuffer->write_32(m_info->key());
+  m_writeBuffer->write_32(m_info->numwant());
+  m_writeBuffer->write_16(manager->connection_manager()->listen_port());
 
   m_writeBuffer->prepare_end();
 
@@ -293,6 +313,7 @@ TrackerUdp::process_announce_output() {
   // connection or not?
   close();
   m_slotSuccess(this, &l);
+
   return true;
 }
   
