@@ -38,7 +38,7 @@
 #define LIBTORRENT_TRACKER_TRACKER_UDP_H
 
 #include "net/protocol_buffer.h"
-#include "net/socket_address.h"
+#include <rak/socket_address.h>
 #include "net/socket_datagram.h"
 #include "globals.h"
 #include "tracker_base.h"
@@ -52,19 +52,19 @@ public:
 
   static const uint64_t magic_connection_id = 0x0000041727101980ll;
 
-  TrackerUdp(TrackerInfo* info, const std::string& url);
+  TrackerUdp(DownloadInfo* info, const std::string& url);
   ~TrackerUdp();
   
   virtual bool        is_busy() const;
 
-  virtual void        send_state(TrackerInfo::State state,
+  virtual void        send_state(DownloadInfo::State state,
 				 uint64_t down,
 				 uint64_t up,
 				 uint64_t left);
 
   virtual void        close();
 
-  virtual Type        get_type() const;
+  virtual Type        type() const;
 
   virtual void        event_read();
   virtual void        event_write();
@@ -83,9 +83,9 @@ private:
   bool                process_announce_output();
   bool                process_error_output();
 
-  SocketAddress       m_connectAddress;
+  rak::socket_address m_connectAddress;
 
-  TrackerInfo::State  m_sendState;
+  DownloadInfo::State m_sendState;
   uint64_t            m_sendUp;
   uint64_t            m_sendDown;
   uint64_t            m_sendLeft;
